"""服务发现模块。

支持多种服务发现方式：
- 配置文件（推荐，通过 BaseConfig.rpc_client.services）
- DNS 解析（统一处理 K8s/Docker Compose，通过服务名自动解析）
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from aury.boot.common.logging import logger

if TYPE_CHECKING:
    from aury.boot.application.config import BaseConfig


class ServiceDiscovery:
    """服务发现抽象基类。
    
    所有服务发现实现都应该继承此类。
    """
    
    def resolve(self, service_name: str) -> str | None:
        """解析服务地址。
        
        Args:
            service_name: 服务名称
            
        Returns:
            str | None: 服务地址（URL），如果无法解析返回 None
        """
        raise NotImplementedError


class ConfigServiceDiscovery(ServiceDiscovery):
    """基于配置文件的服务发现。
    
    从配置文件中读取服务地址映射。
    
    配置格式：
    ```python
    services = {
        "user-service": "http://user-service:8000",
        "order-service": "http://order-service:8001",
    }
    ```
    """
    
    def __init__(self, service_config: dict[str, str] | None = None) -> None:
        """初始化配置服务发现。
        
        Args:
            service_config: 服务配置字典 {service_name: url}
        """
        self._services: dict[str, str] = service_config or {}
    
    def resolve(self, service_name: str) -> str | None:
        """从配置中解析服务地址。"""
        url = self._services.get(service_name)
        if url:
            logger.debug(f"从配置解析服务: {service_name} -> {url}")
            return url
        logger.warning(f"配置中未找到服务: {service_name}")
        return None
    
    def register(self, service_name: str, url: str) -> None:
        """注册服务地址。
        
        Args:
            service_name: 服务名称
            url: 服务地址
        """
        self._services[service_name] = url
        logger.info(f"注册服务: {service_name} -> {url}")




class DNSServiceDiscovery(ServiceDiscovery):
    """DNS 服务发现（统一处理 K8s/Docker Compose）。
    
    直接使用服务名进行 DNS 解析：
    - K8s: 自动使用 service-name（K8s DNS 会在同一 namespace 内解析）
    - Docker Compose: 直接使用 service-name
    """
    
    def __init__(
        self,
        scheme: str = "http",
        default_port: int = 80,
    ) -> None:
        """初始化 DNS 服务发现。
        
        Args:
            scheme: 协议（http 或 https）
            default_port: 默认端口
        """
        self._scheme = scheme
        self._default_port = default_port
    
    def resolve(self, service_name: str) -> str | None:
        """使用 DNS 解析服务地址。
        
        Args:
            service_name: 服务名称（直接作为主机名，DNS 自动解析）
        """
        # 直接使用服务名作为主机名（DNS 自动处理）
        url = f"{self._scheme}://{service_name}"
        
        if self._default_port not in (80, 443):
            url = f"{url}:{self._default_port}"
        
        logger.debug(f"使用 DNS 解析服务: {service_name} -> {url}")
        return url


class CompositeServiceDiscovery(ServiceDiscovery):
    """组合服务发现。
    
    按优先级顺序尝试多种服务发现方式：
    1. 配置文件（最高优先级，通过 BaseConfig.rpc_client.services）
    2. DNS 解析（统一处理 K8s/Docker Compose，自动检测环境）
    """
    
    def __init__(
        self,
        config: "BaseConfig" | None = None,
    ) -> None:
        """初始化组合服务发现。
        
        Args:
            config: 应用配置（BaseConfig），如果为 None，则仅使用默认配置
        """
        self._discoveries: list[ServiceDiscovery] = []
        
        # 从配置中获取 RPC 客户端设置
        if config:
            rpc_client_settings = config.rpc_client
        else:
            # 如果没有提供配置，使用默认配置
            from aury.boot.application.config import RPCClientSettings
            rpc_client_settings = RPCClientSettings()
        
        # 1. 配置文件（最高优先级）
        if rpc_client_settings.services:
            self._discoveries.append(ConfigServiceDiscovery(rpc_client_settings.services))
        
        # 2. DNS 解析（如果启用）
        if rpc_client_settings.use_dns_fallback:
            logger.info("启用 DNS 服务发现（统一处理 K8s/Docker Compose）")
            self._discoveries.append(
                DNSServiceDiscovery(
                    scheme=rpc_client_settings.dns_scheme,
                    default_port=rpc_client_settings.dns_port,
                )
            )
    
    def resolve(self, service_name: str) -> str | None:
        """按优先级顺序解析服务地址。"""
        for discovery in self._discoveries:
            url = discovery.resolve(service_name)
            if url:
                return url
        
        logger.error(f"无法解析服务地址: {service_name}（已尝试所有服务发现方式）")
        return None


# 全局服务发现实例
_global_discovery: ServiceDiscovery | None = None
_global_config: "BaseConfig | None" = None


def get_service_discovery(config: "BaseConfig | None" = None) -> ServiceDiscovery:
    """获取全局服务发现实例。
    
    如果未初始化，使用配置创建组合服务发现。
    
    Args:
        config: 应用配置（可选），如果提供则使用此配置初始化
    
    Returns:
        ServiceDiscovery: 服务发现实例
    """
    global _global_discovery, _global_config
    
    # 如果提供了新配置，更新全局配置
    if config is not None:
        _global_config = config
    
    # 如果未初始化或配置已更新，重新创建
    if _global_discovery is None or config is not None:
        _global_discovery = CompositeServiceDiscovery(_global_config)
    
    return _global_discovery


def set_service_discovery(discovery: ServiceDiscovery) -> None:
    """设置全局服务发现实例。
    
    Args:
        discovery: 服务发现实例
    """
    global _global_discovery
    _global_discovery = discovery
    logger.info("已设置全局服务发现实例")


__all__ = [
    "CompositeServiceDiscovery",
    "ConfigServiceDiscovery",
    "DNSServiceDiscovery",
    "ServiceDiscovery",
    "get_service_discovery",
    "set_service_discovery",
]




"""缓存工厂 - 注册机制。

通过注册机制支持多种缓存后端。
"""

from __future__ import annotations

from typing import ClassVar

from aury.boot.common.logging import logger

from .backends import (
    MemcachedCache,
    MemoryCache,
    RedisCache,
)
from .base import ICache


class CacheFactory:
    """缓存工厂 - 注册机制。
    
    类似Flask-Cache的设计，通过注册机制支持多种后端。
    
    使用示例:
        # 注册后端
        CacheFactory.register("redis", RedisCache)
        CacheFactory.register("memory", MemoryCache)
        
        # 创建缓存实例
        cache = await CacheFactory.create("redis", url="redis://localhost:6379")
        cache = await CacheFactory.create("memory", max_size=1000)
    """
    
    _backends: ClassVar[dict[str, type[ICache]]] = {}
    
    @classmethod
    def register(cls, name: str, backend_class: type[ICache]) -> None:
        """注册缓存后端。
        
        Args:
            name: 后端名称
            backend_class: 后端类
        """
        cls._backends[name] = backend_class
        logger.debug(f"注册缓存后端: {name} -> {backend_class.__name__}")
    
    @classmethod
    async def create(cls, backend_name: str, **config) -> ICache:
        """创建缓存实例。
        
        Args:
            backend_name: 后端名称
            **config: 配置参数
            
        Returns:
            ICache: 缓存实例
            
        Raises:
            ValueError: 后端未注册
        """
        if backend_name not in cls._backends:
            available = ", ".join(cls._backends.keys())
            raise ValueError(
                f"缓存后端 '{backend_name}' 未注册。"
                f"可用后端: {available}"
            )
        
        backend_class = cls._backends[backend_name]
        instance = backend_class(**config)
        
        # 如果后端需要初始化
        if hasattr(instance, "initialize"):
            await instance.initialize()
        
        logger.info(f"创建缓存实例: {backend_name}")
        return instance
    
    @classmethod
    def get_registered(cls) -> list[str]:
        """获取已注册的后端名称。"""
        return list(cls._backends.keys())


# 注册默认后端
CacheFactory.register("redis", RedisCache)
CacheFactory.register("memory", MemoryCache)
CacheFactory.register("memcached", MemcachedCache)


__all__ = [
    "CacheFactory",
]




"""Redis Stream 消息队列后端。

使用 Redis Stream 实现支持消费者组的消息队列。
相比 Redis List，提供更强的持久化和消费保证。
"""

from __future__ import annotations

import asyncio
from collections.abc import Callable
import json
from typing import TYPE_CHECKING, Any

from aury.boot.common.logging import logger

from ..base import IMQ, MQMessage

if TYPE_CHECKING:
    from aury.boot.infrastructure.clients.redis import RedisClient


class RedisStreamMQ(IMQ):
    """Redis Stream 消息队列实现。

    使用 Redis Stream (XADD/XREADGROUP/XACK) 实现可靠的消息队列。
    
    特性:
    - 消费者组支持多实例消费
    - 消息持久化 (配合 AOF)
    - 消息确认机制
    - 支持消息重放
    """

    def __init__(
        self,
        url: str | None = None,
        *,
        redis_client: RedisClient | None = None,
        prefix: str = "stream:",
        consumer_group: str = "default",
        consumer_name: str | None = None,
        max_len: int | None = None,
        max_connections: int = 1000,
    ) -> None:
        """初始化 Redis Stream 消息队列。

        Args:
            url: Redis 连接 URL（当 redis_client 为 None 时必须提供）
            redis_client: RedisClient 实例（可选，优先使用）
            prefix: 队列名称前缀
            consumer_group: 消费者组名称
            consumer_name: 消费者名称（默认自动生成）
            max_len: Stream 最大长度（可选，用于自动裁剪）
            max_connections: Redis 连接池最大连接数（高并发 SSE 场景建议 200+）
        
        Raises:
            ValueError: 当 url 和 redis_client 都为 None 时
        """
        if redis_client is None and url is None:
            raise ValueError("Redis Stream 消息队列需要提供 url 或 redis_client 参数")
        
        self._url = url
        self._client = redis_client
        self._prefix = prefix
        self._consumer_group = consumer_group
        self._consumer_name = consumer_name or f"consumer-{id(self)}"
        self._max_len = max_len
        self._max_connections = max_connections
        self._consuming = False
        self._owns_client = False
        self._log_sample_counter = 0  # 日志采样计数器
    
    # 日志采样率：每 N 个 send 打印 1 次
    LOG_SAMPLE_RATE = 100
    
    async def _ensure_client(self) -> None:
        """确保 Redis 客户端已初始化。"""
        if self._client is None and self._url:
            from aury.boot.infrastructure.clients.redis import RedisClient
            # 创建独立实例（不使用 get_instance 避免和全局实例冲突）
            self._client = RedisClient(name=f"mq-{id(self)}")
            # 使用配置的连接池大小
            self._client.configure(url=self._url, max_connections=self._max_connections)
            await self._client.initialize()
            self._owns_client = True
            logger.info(f"RedisStreamMQ: 创建连接池，max_connections={self._max_connections}")

    def _stream_key(self, queue: str) -> str:
        """获取 Stream 的 Redis key。"""
        return f"{self._prefix}{queue}"

    async def _ensure_group(self, queue: str) -> None:
        """确保消费者组存在。"""
        stream_key = self._stream_key(queue)
        try:
            await self._client.connection.xgroup_create(
                stream_key,
                self._consumer_group,
                id="0",
                mkstream=True,
            )
            logger.debug(f"创建消费者组: {self._consumer_group} on {stream_key}")
        except Exception as e:
            # 组已存在，忽略
            if "BUSYGROUP" not in str(e):
                raise

    async def send(self, queue: str, message: MQMessage) -> str:
        """发送消息到 Stream。
        
        使用 XADD 命令，支持 MAXLEN 自动裁剪。
        """
        await self._ensure_client()
        message.queue = queue
        
        # 序列化消息
        data = {
            "payload": json.dumps(message.to_dict()),
        }
        
        stream_key = self._stream_key(queue)
        
        # XADD with optional MAXLEN
        if self._max_len:
            msg_id = await self._client.connection.xadd(
                stream_key,
                data,
                maxlen=self._max_len,
                approximate=True,  # ~ 近似裁剪，性能更好
            )
        else:
            msg_id = await self._client.connection.xadd(stream_key, data)
        
        # 采样日志：每 N 个消息打印 1 次
        self._log_sample_counter += 1
        if self._log_sample_counter % self.LOG_SAMPLE_RATE == 1:
            logger.debug(f"发送消息到 Stream: {stream_key}, id={msg_id}, count={self._log_sample_counter}")
        return message.id

    async def receive(
        self,
        queue: str,
        timeout: float | None = None,
    ) -> MQMessage | None:
        """从 Stream 接收消息（不使用消费者组）。
        
        用于简单场景，直接 XREAD 读取最新消息。
        """
        await self._ensure_client()
        
        stream_key = self._stream_key(queue)
        timeout_ms = int(timeout * 1000) if timeout else 0
        
        result = await self._client.connection.xread(
            streams={stream_key: "$"},
            count=1,
            block=timeout_ms,
        )
        
        if not result:
            return None
        
        # 解析结果: [[stream_key, [(msg_id, data)]]]
        for stream, messages in result:
            for msg_id, data in messages:
                try:
                    payload = data.get(b"payload") or data.get("payload")
                    if isinstance(payload, bytes):
                        payload = payload.decode()
                    msg_dict = json.loads(payload)
                    message = MQMessage.from_dict(msg_dict)
                    message._stream_id = msg_id  # 保存 stream ID 用于 ACK
                    return message
                except (json.JSONDecodeError, KeyError) as e:
                    logger.error(f"解析消息失败: {e}")
                    return None
        
        return None

    async def receive_group(
        self,
        queue: str,
        timeout: float | None = None,
    ) -> MQMessage | None:
        """从 Stream 接收消息（使用消费者组）。
        
        使用 XREADGROUP 从消费者组读取，支持多实例消费。
        """
        await self._ensure_client()
        await self._ensure_group(queue)
        
        stream_key = self._stream_key(queue)
        timeout_ms = int(timeout * 1000) if timeout else 0
        
        result = await self._client.connection.xreadgroup(
            groupname=self._consumer_group,
            consumername=self._consumer_name,
            streams={stream_key: ">"},  # > 表示只读取新消息
            count=1,
            block=timeout_ms,
        )
        
        if not result:
            return None
        
        # 解析结果
        for stream, messages in result:
            for msg_id, data in messages:
                try:
                    payload = data.get(b"payload") or data.get("payload")
                    if isinstance(payload, bytes):
                        payload = payload.decode()
                    msg_dict = json.loads(payload)
                    message = MQMessage.from_dict(msg_dict)
                    message._stream_id = msg_id  # 保存用于 ACK
                    message.queue = queue
                    return message
                except (json.JSONDecodeError, KeyError) as e:
                    logger.error(f"解析消息失败: {e}")
                    # ACK 损坏的消息，防止阻塞
                    await self._client.connection.xack(
                        stream_key, self._consumer_group, msg_id
                    )
                    return None
        
        return None

    async def ack(self, message: MQMessage) -> None:
        """确认消息已处理。"""
        if not message.queue:
            return
        
        stream_id = getattr(message, "_stream_id", None)
        if stream_id:
            stream_key = self._stream_key(message.queue)
            await self._client.connection.xack(
                stream_key, self._consumer_group, stream_id
            )
            logger.debug(f"ACK 消息: {stream_id}")

    async def nack(self, message: MQMessage, requeue: bool = True) -> None:
        """拒绝消息。
        
        Redis Stream 没有原生 NACK，通过重新发送实现。
        """
        if not message.queue:
            return
        
        stream_id = getattr(message, "_stream_id", None)
        if stream_id:
            stream_key = self._stream_key(message.queue)
            # 先 ACK 原消息
            await self._client.connection.xack(
                stream_key, self._consumer_group, stream_id
            )
            
            if requeue and message.retry_count < message.max_retries:
                # 重新发送
                message.retry_count += 1
                await self.send(message.queue, message)
                logger.debug(f"NACK 重新入队: {message.id}, retry={message.retry_count}")

    async def consume(
        self,
        queue: str,
        handler: Callable[[MQMessage], Any],
        *,
        prefetch: int = 1,
    ) -> None:
        """消费队列消息（使用消费者组）。"""
        self._consuming = True
        await self._ensure_group(queue)
        logger.info(f"开始消费 Stream: {queue}, group={self._consumer_group}")

        while self._consuming:
            try:
                message = await self.receive_group(queue, timeout=1.0)
                if message is None:
                    continue

                try:
                    result = handler(message)
                    if asyncio.iscoroutine(result):
                        await result
                    await self.ack(message)
                except Exception as e:
                    logger.error(f"处理消息失败: {e}")
                    await self.nack(message, requeue=True)

            except Exception as e:
                logger.error(f"消费消息异常: {e}")
                await asyncio.sleep(1)

    async def read_all(
        self,
        queue: str,
        start: str = "-",
        end: str = "+",
        count: int | None = None,
    ) -> list[MQMessage]:
        """读取 Stream 中的所有消息（用于 compaction）。
        
        使用 XRANGE 读取指定范围的消息。
        
        Args:
            queue: 队列名称
            start: 起始 ID（"-" 表示最早）
            end: 结束 ID（"+" 表示最新）
            count: 最大数量
            
        Returns:
            消息列表
        """
        await self._ensure_client()
        stream_key = self._stream_key(queue)
        
        result = await self._client.connection.xrange(
            stream_key,
            min=start,
            max=end,
            count=count,
        )
        
        messages = []
        for msg_id, data in result:
            try:
                payload = data.get(b"payload") or data.get("payload")
                if isinstance(payload, bytes):
                    payload = payload.decode()
                msg_dict = json.loads(payload)
                message = MQMessage.from_dict(msg_dict)
                message._stream_id = msg_id
                messages.append(message)
            except (json.JSONDecodeError, KeyError) as e:
                logger.warning(f"跳过损坏的消息 {msg_id}: {e}")
        
        return messages

    async def read_blocking(
        self,
        queue: str,
        last_id: str = "$",
        count: int = 10,
        block_ms: int = 100,
    ) -> list[MQMessage]:
        """阻塞读取 Stream 中的新消息（使用 XREAD BLOCK）。
        
        Args:
            queue: 队列名称
            last_id: 起始 ID（"$" 表示只等待新消息，"0" 表示从开头）
            count: 最大读取数量
            block_ms: 阻塞等待超时（毫秒），0 表示不阻塞
            
        Returns:
            消息列表
        """
        await self._ensure_client()
        stream_key = self._stream_key(queue)
        
        result = await self._client.connection.xread(
            streams={stream_key: last_id},
            count=count,
            block=block_ms,
        )
        
        if not result:
            return []
        
        messages = []
        for stream, stream_messages in result:
            for msg_id, data in stream_messages:
                try:
                    payload = data.get(b"payload") or data.get("payload")
                    if isinstance(payload, bytes):
                        payload = payload.decode()
                    msg_dict = json.loads(payload)
                    message = MQMessage.from_dict(msg_dict)
                    message._stream_id = msg_id
                    messages.append(message)
                except (json.JSONDecodeError, KeyError) as e:
                    logger.warning(f"跳过损坏的消息 {msg_id}: {e}")
        
        return messages

    async def trim(
        self,
        queue: str,
        *,
        maxlen: int | None = None,
        minid: str | None = None,
    ) -> int:
        """裁剪 Stream。
        
        Args:
            queue: 队列名称
            maxlen: 保留的最大长度
            minid: 保留此 ID 之后的消息
            
        Returns:
            删除的消息数量
        """
        await self._ensure_client()
        stream_key = self._stream_key(queue)
        
        if minid:
            return await self._client.connection.xtrim(
                stream_key, minid=minid, approximate=False
            )
        elif maxlen is not None:
            # maxlen=0 也应该生效（清空 stream）
            return await self._client.connection.xtrim(
                stream_key, maxlen=maxlen, approximate=False
            )
        return 0

    async def delete_stream(self, queue: str) -> bool:
        """删除整个 Stream。"""
        await self._ensure_client()
        stream_key = self._stream_key(queue)
        return await self._client.connection.delete(stream_key) > 0

    async def stream_info(self, queue: str) -> dict[str, Any]:
        """获取 Stream 信息。"""
        await self._ensure_client()
        stream_key = self._stream_key(queue)
        try:
            return await self._client.connection.xinfo_stream(stream_key)
        except Exception:
            return {}

    async def close(self) -> None:
        """关闭连接。"""
        self._consuming = False
        if self._owns_client and self._client:
            await self._client.cleanup()
            self._client = None
        logger.debug("Redis Stream 消息队列已关闭")


__all__ = ["RedisStreamMQ"]



"""基础异常定义。

Common 层是最底层，不依赖任何业务层。
所有框架异常都应该继承 FoundationError。
"""

from __future__ import annotations

from typing import Any


class FoundationError(Exception):
    """基础异常类（最底层）。
    
    所有框架异常都应该继承此类。
    这是整个异常体系的根类。
    
    提供可选的元数据和原因链支持，方便调试和异常包装。
    
    注意：虽然命名为 FoundationError，但它是异常体系的根类，
    所有其他异常都继承它。
    
    Attributes:
        message: 错误消息
        metadata: 可选的元数据字典，用于存储额外的上下文信息
        cause: 可选的原始异常，用于异常链
    
    使用示例:
        # 基本用法
        raise FoundationError("操作失败")
        
        # 带元数据
        raise FoundationError(
            "操作失败",
            metadata={"user_id": 123, "operation": "update"}
        )
        
        # 包装其他异常
        try:
            risky_operation()
        except ValueError as e:
            raise FoundationError(
                "操作失败",
                cause=e,
                metadata={"context": "data_processing"}
            ) from e
    """
    
    def __init__(
        self,
        message: str,
        *args: object,
        metadata: dict[str, Any] | None = None,
        cause: Exception | None = None,
    ) -> None:
        """初始化异常。
        
        Args:
            message: 错误消息
            *args: 其他参数（传递给父类）
            metadata: 可选的元数据字典，用于存储额外的上下文信息
            cause: 可选的原始异常，用于异常链
        """
        self.message = message
        self.metadata = metadata or {}
        self.cause = cause
        
        super().__init__(message, *args)
        
        # 如果提供了 cause，设置异常链
        if cause is not None:
            self.__cause__ = cause
    
    def __str__(self) -> str:
        """返回异常字符串表示。"""
        return self.message
    
    def __repr__(self) -> str:
        """返回异常的详细表示，包含元数据信息。"""
        parts = [f"{self.__class__.__name__}(message={self.message!r}"]
        
        if self.metadata:
            parts.append(f", metadata={self.metadata}")
        
        if self.cause:
            parts.append(f", cause={self.cause.__class__.__name__}")
        
        parts.append(")")
        return "".join(parts)
    
    def with_metadata(self, **kwargs: Any) -> FoundationError:
        """添加或更新元数据（链式调用）。
        
        Args:
            **kwargs: 要添加的元数据键值对
        
        Returns:
            FoundationError: 返回自身，支持链式调用
        
        使用示例:
            raise FoundationError("操作失败").with_metadata(
                user_id=123,
                operation="update"
            )
        """
        self.metadata.update(kwargs)
        return self


__all__ = [
    "FoundationError",
]





"""告警事件定义。

定义告警事件类型、严重级别和事件数据结构。
"""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Any
import hashlib


class AlertEventType(str, Enum):
    """告警事件类型。"""
    
    SLOW_REQUEST = "slow_request"       # 慢请求
    SLOW_SQL = "slow_sql"               # 慢 SQL
    EXCEPTION = "exception"             # 异常
    TASK_FAILURE = "task_failure"       # 任务失败
    TASK_TIMEOUT = "task_timeout"       # 任务超时
    CUSTOM = "custom"                   # 自定义


class AlertSeverity(str, Enum):
    """告警严重级别。"""
    
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"
    CRITICAL = "critical"


@dataclass
class AlertEvent:
    """告警事件。
    
    包含告警的所有上下文信息。
    """
    
    event_type: AlertEventType
    severity: AlertSeverity
    message: str
    trace_id: str
    
    source: str = "unknown"           # api / scheduler / task
    service_name: str = ""            # 服务名
    timestamp: datetime = field(default_factory=datetime.now)
    metadata: dict[str, Any] = field(default_factory=dict)
    
    # 用于聚合/去重的指纹（相同指纹的事件会被聚合）
    _fingerprint: str | None = field(default=None, repr=False)
    
    @property
    def fingerprint(self) -> str:
        """获取事件指纹。
        
        相同指纹的事件会被聚合处理。
        """
        if self._fingerprint:
            return self._fingerprint
        
        # 默认指纹：类型 + 来源 + 关键元数据
        key_parts = [
            self.event_type.value,
            self.source,
            self.metadata.get("endpoint", ""),
            self.metadata.get("task_name", ""),
            self.metadata.get("error_type", ""),
        ]
        key_str = ":".join(str(p) for p in key_parts if p)
        return hashlib.md5(key_str.encode()).hexdigest()[:16]
    
    @fingerprint.setter
    def fingerprint(self, value: str) -> None:
        self._fingerprint = value
    
    def to_dict(self) -> dict[str, Any]:
        """转换为字典。"""
        return {
            "event_type": self.event_type.value,
            "severity": self.severity.value,
            "message": self.message,
            "trace_id": self.trace_id,
            "source": self.source,
            "service_name": self.service_name,
            "timestamp": self.timestamp.isoformat(),
            "metadata": self.metadata,
            "fingerprint": self.fingerprint,
        }


@dataclass
class AlertNotification:
    """告警通知（发送给 Notifier 的数据）。
    
    包含聚合后的告警信息。
    """
    
    title: str
    message: str
    severity: AlertSeverity
    event_type: AlertEventType
    source: str
    service_name: str
    
    # 聚合信息
    count: int = 1                    # 聚合的事件数量
    first_timestamp: datetime = field(default_factory=datetime.now)
    last_timestamp: datetime = field(default_factory=datetime.now)
    
    # 关联的 trace_id 列表（最多保留最近几个）
    trace_ids: list[str] = field(default_factory=list)
    
    # 额外元数据
    metadata: dict[str, Any] = field(default_factory=dict)
    
    def to_dict(self) -> dict[str, Any]:
        """转换为字典。"""
        return {
            "title": self.title,
            "message": self.message,
            "severity": self.severity.value,
            "event_type": self.event_type.value,
            "source": self.source,
            "service_name": self.service_name,
            "count": self.count,
            "first_timestamp": self.first_timestamp.isoformat(),
            "last_timestamp": self.last_timestamp.isoformat(),
            "trace_ids": self.trace_ids,
            "metadata": self.metadata,
        }


__all__ = [
    "AlertEvent",
    "AlertEventType",
    "AlertNotification",
    "AlertSeverity",
]



"""基础设施层模块。

提供外部依赖的实现，包括：
- 数据库管理
- 缓存管理
- 存储管理
- 调度器
- 任务队列
- 消息队列
- 通道 (SSE/PubSub)
- 事件总线
- Redis 客户端
- RabbitMQ 客户端
"""

# 数据库
# 缓存
from .cache import (
    CacheBackend,
    CacheFactory,
    CacheManager,
    ICache,
    MemcachedCache,
    MemoryCache,
    RedisCache,
)

# 通道 (SSE/PubSub)
from .channel import (
    BroadcasterChannel,
    ChannelBackend,
    ChannelManager,
    ChannelMessage,
    IChannel,
)

# RabbitMQ 客户端
from .clients.rabbitmq import RabbitMQClient, RabbitMQConfig

# Redis 客户端
from .clients.redis import RedisClient, RedisConfig
from .database import DatabaseManager

# 依赖注入
from .di import Container, Lifetime, Scope, ServiceDescriptor

# 事件总线
from .events import (
    BroadcasterEventBus,
    Event,
    EventBackend,
    EventBusManager,
    EventHandler,
    EventType,
    IEventBus,
    RabbitMQEventBus,
)

# 消息队列
from .mq import (
    IMQ,
    MQBackend,
    MQManager,
    MQMessage,
    RabbitMQ,
    RedisMQ,
)

# 存储（基于 aury-sdk-storage）
from .storage import (
    IStorage,
    LocalStorage,
    S3Storage,
    StorageBackend,
    StorageConfig,
    StorageFactory,
    StorageFile,
    StorageManager,
    UploadResult,
)

# 调度器（可选依赖）
try:
    from .scheduler import SchedulerManager
except ImportError:
    SchedulerManager = None  # type: ignore[assignment, misc]

# 任务队列（可选依赖）
try:
    from .tasks import TaskManager, TaskProxy, conditional_task
except ImportError:
    TaskManager = None  # type: ignore[assignment, misc]
    TaskProxy = None  # type: ignore[assignment, misc]
    conditional_task = None  # type: ignore[assignment, misc]

__all__ = [
    # 消息队列
    "IMQ",
    # 缓存
    "CacheBackend",
    "CacheFactory",
    "CacheManager",
    # 通道
    "BroadcasterChannel",
    "ChannelBackend",
    "ChannelManager",
    "ChannelMessage",
    # 依赖注入
    "Container",
    # 数据库
    "DatabaseManager",
    # 事件总线
    "Event",
    "EventBackend",
    "EventBusManager",
    "EventHandler",
    "EventType",
    "ICache",
    "IChannel",
    "IEventBus",
    # 存储
    "IStorage",
    "Lifetime",
    "LocalStorage",
    "MQBackend",
    "MQManager",
    "MQMessage",
    "MemcachedCache",
    "MemoryCache",
    "RabbitMQ",
    # RabbitMQ 客户端
    "RabbitMQClient",
    "RabbitMQConfig",
    "RabbitMQEventBus",
    "BroadcasterEventBus",
    "RedisCache",
    # Redis 客户端
    "RedisClient",
    "RedisConfig",
    "RedisMQ",
    "S3Storage",
    # 调度器
    "SchedulerManager",
    "Scope",
    "ServiceDescriptor",
    "StorageBackend",
    "StorageConfig",
    "StorageFactory",
    "StorageFile",
    "StorageManager",
    # 任务队列
    "TaskManager",
    "TaskProxy",
    "UploadResult",
    "conditional_task",
]




"""数据库配置。

Infrastructure 层配置数据类，由 application 层传入。
"""

from __future__ import annotations

from pydantic import BaseModel, Field

# 支持的事务隔离级别
ISOLATION_LEVELS = (
    "READ UNCOMMITTED",
    "READ COMMITTED",
    "REPEATABLE READ",
    "SERIALIZABLE",
    "AUTOCOMMIT",
)


class DatabaseConfig(BaseModel):
    """数据库基础设施配置。
    
    纯数据类，由 application 层构造并传入 infrastructure 层。
    不直接读取环境变量。
    """
    
    url: str = Field(
        description="数据库连接URL（支持所有 SQLAlchemy 支持的数据库）"
    )
    echo: bool = Field(
        default=False,
        description="是否打印SQL语句"
    )
    pool_size: int = Field(
        default=5,
        description="连接池大小"
    )
    max_overflow: int = Field(
        default=10,
        description="最大溢出连接数"
    )
    pool_timeout: int = Field(
        default=30,
        description="连接超时时间（秒）"
    )
    pool_recycle: int = Field(
        default=1800,
        description="连接回收时间（秒）"
    )
    isolation_level: str | None = Field(
        default=None,
        description="事务隔离级别: READ UNCOMMITTED / READ COMMITTED / REPEATABLE READ / SERIALIZABLE / AUTOCOMMIT"
    )


__all__ = [
    "ISOLATION_LEVELS",
    "DatabaseConfig",
]






"""告警管理器。

核心告警处理逻辑，包括：
- 规则匹配
- 事件聚合
- 通知发送
"""

from __future__ import annotations

import asyncio
from pathlib import Path
from typing import TYPE_CHECKING, Any

from aury.boot.common.logging import logger

from .aggregator import AlertAggregator
from .events import AlertEvent, AlertEventType, AlertNotification, AlertSeverity
from .rules import AlertRule, load_rules_from_dict

if TYPE_CHECKING:
    from .notifiers.base import AlertNotifier


class AlertManager:
    """告警管理器。
    
    负责处理告警事件、匹配规则、聚合事件、发送通知。
    
    使用方式：
        # 初始化（通常在应用启动时）
        alert_manager = AlertManager.get_instance()
        await alert_manager.initialize(config)
        
        # 发送告警事件
        await alert_manager.emit(AlertEvent(...))
        
        # 或使用便捷函数
        await emit_alert(AlertEventType.SLOW_REQUEST, "慢请求", duration=1.5)
    """
    
    _instance: "AlertManager | None" = None
    _notifier_classes: dict[str, type["AlertNotifier"]] = {}
    
    def __init__(self) -> None:
        """初始化告警管理器。"""
        self._enabled = False
        self._service_name = ""
        self._rules: list[AlertRule] = []
        self._notifiers: dict[str, "AlertNotifier"] = {}
        self._aggregators: dict[str, AlertAggregator] = {}  # 每个规则一个聚合器
        self._defaults: dict[str, Any] = {}
        self._initialized = False
    
    @classmethod
    def get_instance(cls) -> "AlertManager":
        """获取单例实例。"""
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance
    
    @classmethod
    def register_notifier_class(cls, name: str, notifier_cls: type["AlertNotifier"]) -> None:
        """注册通知器类型。
        
        Args:
            name: 类型名称（如 "feishu", "webhook"）
            notifier_cls: 通知器类
        """
        cls._notifier_classes[name] = notifier_cls
    
    async def initialize(
        self,
        *,
        enabled: bool = True,
        service_name: str = "",
        rules_file: str | Path | None = None,
        defaults: dict[str, Any] | None = None,
        notifiers: dict[str, dict[str, Any]] | None = None,
    ) -> None:
        """初始化告警管理器。
        
        Args:
            enabled: 是否启用告警
            service_name: 服务名称
            rules_file: 规则文件路径（YAML）
            defaults: 默认配置
            notifiers: 通知器配置（从 config.alert.get_notifiers() 获取）
        """
        self._enabled = enabled
        self._service_name = service_name
        self._defaults = defaults or {}
        
        if not enabled:
            logger.info("告警系统已禁用")
            return
        
        # 注册内置通知器类型
        self._register_builtin_notifiers()
        
        # 从配置加载内置通知器
        if notifiers:
            self._load_notifiers_from_config(notifiers)
        
        # 加载规则
        if rules_file:
            await self._load_rules_from_file(rules_file)
        
        # 如果没有规则，创建默认规则
        if not self._rules:
            self._create_default_rules()
        
        # 为每个规则创建聚合器
        for rule in self._rules:
            self._aggregators[rule.name] = AlertAggregator(
                window_seconds=rule.aggregate_window,
                threshold=rule.aggregate_threshold,
                suppress_seconds=rule.suppress_seconds,
            )
        
        self._initialized = True
        logger.info(
            f"告警系统已初始化: {len(self._rules)} 条规则, "
            f"{len(self._notifiers)} 个通知器"
        )
    
    def _register_builtin_notifiers(self) -> None:
        """注册内置通知器类型。"""
        from .notifiers.feishu import FeishuNotifier
        from .notifiers.webhook import WebhookNotifier
        
        self.register_notifier_class("feishu", FeishuNotifier)
        self.register_notifier_class("webhook", WebhookNotifier)
    
    def _load_notifiers_from_config(
        self,
        notifiers: dict[str, dict[str, Any]],
    ) -> None:
        """从配置加载通知器。
        
        Args:
            notifiers: 通知器配置（从 config.alert.get_notifiers() 获取）
        """
        for name, config in notifiers.items():
            # type 字段决定通知器类型，默认用实例名
            notifier_type = config.pop("type", name)
            notifier_cls = self._notifier_classes.get(notifier_type)
            
            if not notifier_cls:
                logger.warning(f"未知的通知器类型: {notifier_type}，跳过 {name}")
                continue
            
            try:
                notifier = notifier_cls.from_config(config)
                self._notifiers[name] = notifier
                logger.debug(f"已加载通知器: {name} ({notifier_type})")
            except Exception as e:
                logger.error(f"加载通知器 {name} 失败: {e}")
        
        # 如果有 notifier，将第一个设为 default
        if self._notifiers and "default" not in self._notifiers:
            first_name = next(iter(self._notifiers))
            self._notifiers["default"] = self._notifiers[first_name]
    
    async def _load_rules_from_file(self, rules_file: str | Path) -> None:
        """从 YAML 文件加载规则。"""
        rules_path = Path(rules_file)
        if not rules_path.exists():
            logger.warning(f"规则文件不存在: {rules_path}")
            return
        
        try:
            import yaml
            with open(rules_path, encoding="utf-8") as f:
                data = yaml.safe_load(f)
            
            if data:
                defaults, rules = load_rules_from_dict(data)
                self._defaults.update(defaults)
                self._rules.extend(rules)
                logger.info(f"从 {rules_path} 加载了 {len(rules)} 条规则")
        except ImportError:
            logger.warning("未安装 PyYAML，无法加载 YAML 规则文件")
        except Exception as e:
            logger.error(f"加载规则文件失败: {e}")
    
    def _create_default_rules(self) -> None:
        """创建默认规则。"""
        slow_request_threshold = self._defaults.get("slow_request_threshold", 1.0)
        slow_sql_threshold = self._defaults.get("slow_sql_threshold", 0.5)
        slow_request_exclude_paths = self._defaults.get("slow_request_exclude_paths") or None
        
        default_rules = [
            # 慢请求
            AlertRule(
                name="default_slow_request",
                event_types=[AlertEventType.SLOW_REQUEST],
                threshold=slow_request_threshold,
                aggregate_window=self._defaults.get("aggregate_window", 10),
                aggregate_threshold=self._defaults.get("slow_request_aggregate", 5),
                suppress_seconds=self._defaults.get("suppress_seconds", 300),
                exclude_paths=slow_request_exclude_paths,
            ),
            # 慢 SQL
            AlertRule(
                name="default_slow_sql",
                event_types=[AlertEventType.SLOW_SQL],
                threshold=slow_sql_threshold,
                aggregate_window=self._defaults.get("aggregate_window", 10),
                aggregate_threshold=self._defaults.get("slow_sql_aggregate", 5),
                suppress_seconds=self._defaults.get("suppress_seconds", 300),
            ),
            # 异常（立即告警）
            AlertRule(
                name="default_exception",
                event_types=[AlertEventType.EXCEPTION],
                aggregate_threshold=self._defaults.get("exception_aggregate", 1),
                suppress_seconds=self._defaults.get("suppress_seconds", 300),
            ),
            # 任务失败（立即告警）
            AlertRule(
                name="default_task_failure",
                event_types=[AlertEventType.TASK_FAILURE],
                aggregate_threshold=1,
                suppress_seconds=60,
            ),
            # 任务超时
            AlertRule(
                name="default_task_timeout",
                event_types=[AlertEventType.TASK_TIMEOUT],
                aggregate_threshold=1,
                suppress_seconds=300,
            ),
            # 自定义告警（立即告警）
            AlertRule(
                name="default_custom",
                event_types=[AlertEventType.CUSTOM],
                aggregate_threshold=1,
                suppress_seconds=self._defaults.get("suppress_seconds", 10),
            ),
        ]
        
        self._rules.extend(default_rules)
    
    def add_rule(self, rule: AlertRule) -> None:
        """添加告警规则。
        
        Args:
            rule: 告警规则
        """
        self._rules.append(rule)
        self._aggregators[rule.name] = AlertAggregator(
            window_seconds=rule.aggregate_window,
            threshold=rule.aggregate_threshold,
            suppress_seconds=rule.suppress_seconds,
        )
    
    def register_notifier(self, name: str, notifier: "AlertNotifier") -> None:
        """注册通知器实例。
        
        Args:
            name: 通知器名称
            notifier: 通知器实例
        """
        self._notifiers[name] = notifier
    
    async def emit(self, event: AlertEvent) -> None:
        """发送告警事件。
        
        Args:
            event: 告警事件
        """
        if not self._enabled:
            return
        
        if not self._initialized:
            logger.warning("告警系统未初始化，跳过事件")
            return
        
        # 设置服务名
        if not event.service_name:
            event.service_name = self._service_name
        
        # 匹配规则
        for rule in self._rules:
            if rule.matches(event):
                aggregator = self._aggregators.get(rule.name)
                if aggregator and aggregator.should_alert(event):
                    await self._send_notification(rule, event, aggregator)
                break  # 只匹配第一个规则
    
    async def _send_notification(
        self,
        rule: AlertRule,
        event: AlertEvent,
        aggregator: AlertAggregator,
    ) -> None:
        """发送通知。"""
        # 获取聚合信息
        agg_info = aggregator.get_aggregation_info(event)
        
        # 构建通知
        notification = AlertNotification(
            title=self._build_title(event, agg_info["count"]),
            message=event.message,
            severity=event.severity,
            event_type=event.event_type,
            source=event.source,
            service_name=event.service_name,
            count=agg_info["count"],
            trace_ids=agg_info["trace_ids"],
            metadata=event.metadata,
        )
        
        # 发送到所有配置的通知器
        for notifier_name in rule.notifiers:
            notifier = self._notifiers.get(notifier_name)
            if notifier:
                # 异步发送，不阻塞
                asyncio.create_task(self._safe_send(notifier, notification))
            else:
                logger.warning(f"通知器不存在: {notifier_name}")
    
    async def _safe_send(self, notifier: "AlertNotifier", notification: AlertNotification) -> None:
        """安全发送通知（捕获异常）。"""
        try:
            await notifier.send(notification)
        except Exception as e:
            logger.error(f"发送通知失败: {e}")
    
    def _build_title(self, event: AlertEvent, count: int) -> str:
        """构建通知标题。"""
        type_names = {
            AlertEventType.SLOW_REQUEST: "慢请求",
            AlertEventType.SLOW_SQL: "慢SQL",
            AlertEventType.EXCEPTION: "异常",
            AlertEventType.TASK_FAILURE: "任务失败",
            AlertEventType.TASK_TIMEOUT: "任务超时",
            AlertEventType.CUSTOM: "告警",
        }
        type_name = type_names.get(event.event_type, "告警")
        
        if count > 1:
            return f"[{event.severity.value.upper()}] {type_name} x{count}"
        return f"[{event.severity.value.upper()}] {type_name}"
    
    @property
    def is_enabled(self) -> bool:
        """是否启用。"""
        return self._enabled
    
    @property
    def is_initialized(self) -> bool:
        """是否已初始化。"""
        return self._initialized


# 便捷函数
async def emit_alert(
    event_type: AlertEventType,
    message: str,
    *,
    severity: AlertSeverity = AlertSeverity.WARNING,
    trace_id: str | None = None,
    source: str | None = None,
    **metadata: Any,
) -> None:
    """发送告警事件的便捷函数。
    
    自动获取 trace_id 和检测来源。
    
    Args:
        event_type: 事件类型
        message: 告警消息
        severity: 严重级别
        trace_id: 追踪 ID（可选，自动获取）
        source: 来源（可选，自动检测）
        **metadata: 额外元数据
    
    示例：
        await emit_alert(
            AlertEventType.SLOW_SQL,
            "慢SQL查询",
            duration=2.5,
            sql="SELECT ...",
        )
    """
    from aury.boot.common.logging import get_trace_id
    
    if trace_id is None:
        trace_id = get_trace_id() or ""
    
    if source is None:
        source = _detect_source()
    
    event = AlertEvent(
        event_type=event_type,
        severity=severity,
        message=message,
        trace_id=trace_id,
        source=source,
        metadata=metadata,
    )
    
    manager = AlertManager.get_instance()
    await manager.emit(event)


def _detect_source() -> str:
    """检测当前执行来源。"""
    import inspect
    
    # 检查调用栈
    for frame_info in inspect.stack():
        module = frame_info.frame.f_globals.get("__name__", "")
        
        if "scheduler" in module.lower():
            return "scheduler"
        if "task" in module.lower() or "worker" in module.lower():
            return "task"
        if "middleware" in module.lower() or "api" in module.lower():
            return "api"
    
    return "unknown"


async def emit_exception_alert(
    exc: Exception,
    message: str | None = None,
    *,
    severity: AlertSeverity = AlertSeverity.ERROR,
    source: str | None = None,
    **metadata: Any,
) -> None:
    """发送异常告警的便捷函数（带完整堆栈）。
    
    用于在 try/except 中捕获异常后手动发送告警。
    框架只会自动告警未被捕获的异常，被 catch 的异常需要手动调用此函数。
    
    Args:
        exc: 异常对象
        message: 告警消息（默认使用异常信息）
        severity: 严重级别（默认 ERROR）
        source: 来源（可选，自动检测）
        **metadata: 额外元数据（如 session_id, user_id 等）
        
    Example:
        try:
            await some_operation()
        except Exception as e:
            await emit_exception_alert(
                e, 
                source="task_stream",
                session_id=session_id,
            )
            # 继续处理...
    """
    import traceback
    
    exc_type = type(exc).__name__
    exc_msg = str(exc)
    tb = traceback.format_exc()
    
    alert_message = message or f"{exc_type}: {exc_msg}"
    
    # 合并异常信息到 metadata
    metadata["error_type"] = exc_type
    metadata["error_message"] = exc_msg
    metadata["stacktrace"] = tb
    
    await emit_alert(
        AlertEventType.EXCEPTION,
        alert_message,
        severity=severity,
        source=source,
        **metadata,
    )


__all__ = [
    "AlertManager",
    "emit_alert",
    "emit_exception_alert",
]



"""数据库管理器 - 命名多实例实现。

提供统一的数据库连接管理、会话创建和健康检查功能。
"""

from __future__ import annotations

import asyncio
from collections.abc import AsyncGenerator
from contextlib import asynccontextmanager

from sqlalchemy import text
from sqlalchemy.exc import DisconnectionError, OperationalError, SQLAlchemyError
from sqlalchemy.ext.asyncio import AsyncEngine, AsyncSession, async_sessionmaker, create_async_engine

from aury.boot.common.logging import logger


class DatabaseManager:
    """数据库管理器（命名多实例）。
    
    职责：
    1. 管理数据库引擎和连接池
    2. 提供会话工厂
    3. 健康检查和重连机制
    4. 生命周期管理
    5. 支持多个命名实例，如主库/从库、不同业务数据库等
    
    使用示例:
        # 默认实例
        db_manager = DatabaseManager.get_instance()
        await db_manager.initialize()
        
        # 命名实例
        primary = DatabaseManager.get_instance("primary")
        replica = DatabaseManager.get_instance("replica")
        
        # 获取会话
        async with db_manager.session() as session:
            # 使用 session 进行数据库操作
            pass
        
        # 清理
        await db_manager.cleanup()
    """
    
    _instances: dict[str, DatabaseManager] = {}
    
    def __init__(self, name: str = "default") -> None:
        """初始化数据库管理器。
        
        Args:
            name: 实例名称
        """
        self.name = name
        self._initialized: bool = False
        self._engine: AsyncEngine | None = None
        self._session_factory: async_sessionmaker | None = None
        self._max_retries: int = 3
        self._retry_delay: float = 1.0
    
    @classmethod
    def get_instance(cls, name: str = "default") -> DatabaseManager:
        """获取指定名称的实例。
        
        Args:
            name: 实例名称，默认为 "default"
            
        Returns:
            DatabaseManager: 数据库管理器实例
        """
        if name not in cls._instances:
            cls._instances[name] = cls(name)
        return cls._instances[name]
    
    @classmethod
    def reset_instance(cls, name: str | None = None) -> None:
        """重置实例（仅用于测试）。
        
        Args:
            name: 要重置的实例名称。如果为 None，则重置所有实例。
            
        注意：调用此方法前应先调用 cleanup() 释放资源。
        """
        if name is None:
            cls._instances.clear()
        elif name in cls._instances:
            del cls._instances[name]
    
    @property
    def engine(self) -> AsyncEngine:
        """获取数据库引擎。"""
        if self._engine is None:
            raise RuntimeError("数据库管理器未初始化，请先调用 initialize()")
        return self._engine
    
    @property
    def session_factory(self) -> async_sessionmaker:
        """获取会话工厂。"""
        if self._session_factory is None:
            raise RuntimeError("数据库管理器未初始化，请先调用 initialize()")
        return self._session_factory
    
    @property
    def is_initialized(self) -> bool:
        """检查是否已初始化。"""
        return self._initialized
    
    async def initialize(
        self,
        url: str | None = None,
        *,
        echo: bool | None = None,
        pool_size: int | None = None,
        max_overflow: int | None = None,
        pool_timeout: int | None = None,
        pool_recycle: int | None = None,
        isolation_level: str | None = None,
    ) -> None:
        """初始化数据库连接。
        
        Args:
            url: 数据库连接字符串，默认从配置读取
            echo: 是否打印SQL语句
            pool_size: 连接池大小
            max_overflow: 最大溢出连接数
            pool_timeout: 连接超时时间（秒）
            pool_recycle: 连接回收时间（秒）
            isolation_level: 事务隔离级别
        """
        if self._initialized:
            logger.warning("数据库管理器已初始化，跳过重复初始化")
            return
        
        # 使用提供的参数或环境变量默认值
        import os
        database_url = url or os.getenv("DATABASE_URL")
        if not database_url:
            raise ValueError(
                "数据库 URL 未配置。请通过以下方式之一提供："
                "1. 通过 initialize(url=...) 参数传入"
                "2. 设置环境变量 DATABASE_URL"
            )
        db_echo = echo if echo is not None else os.getenv("DB_ECHO", "false").lower() == "true"
        db_pool_size = pool_size or int(os.getenv("DB_POOL_SIZE", "5"))
        db_max_overflow = max_overflow or int(os.getenv("DB_MAX_OVERFLOW", "10"))
        db_pool_timeout = pool_timeout or int(os.getenv("DB_POOL_TIMEOUT", "30"))
        db_pool_recycle = pool_recycle or int(os.getenv("DB_POOL_RECYCLE", "1800"))
        db_isolation_level = isolation_level or os.getenv("DATABASE_ISOLATION_LEVEL")
        
        # 构建引擎参数
        engine_kwargs: dict = {
            "echo": db_echo,
            "future": True,
            "pool_pre_ping": True,
            "pool_size": db_pool_size,
            "max_overflow": db_max_overflow,
            "pool_timeout": db_pool_timeout,
            "pool_recycle": db_pool_recycle,
        }
        
        # 添加隔离级别（如果配置了）
        if db_isolation_level:
            engine_kwargs["isolation_level"] = db_isolation_level
            logger.info(f"事务隔离级别设置为: {db_isolation_level}")
        
        self._engine = create_async_engine(database_url, **engine_kwargs)
        
        self._session_factory = async_sessionmaker(
            self._engine,
            expire_on_commit=False,
            autoflush=False,
            class_=AsyncSession,
        )
        
        # 验证连接
        await self.health_check()
        
        self._initialized = True
        logger.info("数据库管理器初始化完成")
    
    async def health_check(self) -> bool:
        """健康检查。
        
        Returns:
            bool: 连接是否正常
        """
        try:
            async with self.engine.begin() as conn:
                await conn.execute(text("SELECT 1"))
            logger.debug("数据库健康检查通过")
            return True
        except Exception as exc:
            logger.error(f"数据库健康检查失败: {exc}")
            return False
    
    async def _check_session_connection(self, session: AsyncSession) -> None:
        """检查会话连接状态，必要时重连。
        
        使用 SQLAlchemy 的通用异常类，支持所有数据库后端。
        
        Args:
            session: 数据库会话
            
        Raises:
            Exception: 重试次数耗尽后抛出异常
        """
        retries = self._max_retries
        while retries > 0:
            try:
                await session.execute(text("SELECT 1"))
                return
            except (DisconnectionError, OperationalError) as exc:
                logger.warning(f"数据库连接丢失，剩余重试次数: {retries}, 错误: {exc}")
                retries -= 1
                if retries == 0:
                    logger.error("数据库连接重试失败")
                    raise
                await asyncio.sleep(self._retry_delay)
            except Exception as exc:
                # 其他异常直接抛出，不重试
                logger.error(f"数据库连接检查失败: {exc}")
                raise
    
    @asynccontextmanager
    async def session(self) -> AsyncGenerator[AsyncSession]:
        """获取数据库会话（上下文管理器）。
        
        连接校验由 pool_pre_ping=True 在引擎层自动处理，
        无需手动检查。
        
        Yields:
            AsyncSession: 数据库会话
            
        使用示例:
            async with db_manager.session() as session:
                result = await session.execute(query)
        """
        session = self.session_factory()
        try:
            yield session
        except SQLAlchemyError as exc:
            # 只捕获数据库相关异常
            await session.rollback()
            logger.exception(f"数据库会话异常: {exc}")
            raise
        except Exception:
            # 非数据库异常（如请求验证错误）：仍需回滚以确保事务一致性
            # 但不记录为数据库异常，直接传播
            await session.rollback()
            raise
        finally:
            await session.close()
    
    async def create_session(self) -> AsyncSession:
        """创建新的数据库会话（需要手动关闭）。
        
        连接校验由 pool_pre_ping=True 在引擎层自动处理。
        
        Returns:
            AsyncSession: 数据库会话
            
        注意：使用后需要手动调用 await session.close()
        建议使用 session() 上下文管理器代替此方法。
        """
        return self.session_factory()
    
    async def get_session(self) -> AsyncGenerator[AsyncSession]:
        """FastAPI 依赖注入专用的会话获取器。
        
        Yields:
            AsyncSession: 数据库会话
            
        使用示例（FastAPI 路由）:
            @router.get("/items")
            async def get_items(
                session: AsyncSession = Depends(db_manager.get_session),
            ):
                ...
        """
        async with self.session() as session:
            yield session
    
    async def cleanup(self) -> None:
        """清理资源，关闭所有连接。"""
        if self._engine is not None:
            await self._engine.dispose()
            logger.info("数据库连接已关闭")
        
        self._engine = None
        self._session_factory = None
        self._initialized = False
    
    def __repr__(self) -> str:
        """字符串表示。"""
        status = "initialized" if self._initialized else "not initialized"
        return f"<DatabaseManager status={status}>"


__all__ = [
    "DatabaseManager",
]




"""通用 Webhook 通知器。

通过 HTTP POST 发送告警到任意 Webhook 端点。
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any

import aiohttp

from aury.boot.common.logging import logger

from .base import AlertNotifier

if TYPE_CHECKING:
    from ..events import AlertNotification


class WebhookNotifier(AlertNotifier):
    """通用 Webhook 通知器。
    
    将告警以 JSON 格式 POST 到指定 URL。
    支持自定义请求头。
    
    环境变量配置示例：
        ALERT_NOTIFIER_MYWEBHOOK_TYPE=webhook
        ALERT_NOTIFIER_MYWEBHOOK_URL=https://my-alert-system.com/api/alert
        ALERT_NOTIFIER_MYWEBHOOK_HEADERS={"Authorization": "Bearer xxx"}
    """
    
    def __init__(
        self,
        url: str,
        headers: dict[str, str] | None = None,
        timeout: int = 10,
    ) -> None:
        """初始化 Webhook 通知器。
        
        Args:
            url: Webhook URL
            headers: 自定义请求头
            timeout: 请求超时时间（秒）
        """
        self.url = url
        self.headers = headers or {}
        self.timeout = timeout
    
    @classmethod
    def from_config(cls, config: dict) -> "WebhookNotifier":
        """从配置创建实例。"""
        url = config.get("url")
        if not url:
            raise ValueError("Webhook 通知器配置缺少 url")
        
        # 解析 headers（可能是 JSON 字符串）
        headers = config.get("headers")
        if isinstance(headers, str):
            import json
            try:
                headers = json.loads(headers)
            except json.JSONDecodeError:
                headers = {}
        
        timeout = int(config.get("timeout", 10))
        
        return cls(url=url, headers=headers, timeout=timeout)
    
    def _build_payload(self, notification: "AlertNotification") -> dict[str, Any]:
        """构建请求体。"""
        return {
            "title": notification.title,
            "message": notification.message,
            "severity": notification.severity.value,
            "event_type": notification.event_type.value,
            "source": notification.source,
            "service_name": notification.service_name,
            "count": notification.count,
            "first_timestamp": notification.first_timestamp.isoformat(),
            "last_timestamp": notification.last_timestamp.isoformat(),
            "trace_ids": notification.trace_ids,
            "metadata": notification.metadata,
        }
    
    async def send(self, notification: "AlertNotification") -> bool:
        """发送 Webhook 通知。"""
        try:
            payload = self._build_payload(notification)
            
            timeout = aiohttp.ClientTimeout(total=self.timeout)
            async with aiohttp.ClientSession(timeout=timeout) as session:
                async with session.post(
                    self.url,
                    json=payload,
                    headers=self.headers,
                ) as response:
                    if response.status < 400:
                        logger.debug(f"Webhook 通知发送成功: {notification.title}")
                        return True
                    else:
                        text = await response.text()
                        logger.error(
                            f"Webhook 通知发送失败: {response.status} - {text}"
                        )
                        return False
        except Exception as e:
            logger.error(f"Webhook 通知发送异常: {e}")
            return False


__all__ = ["WebhookNotifier"]



"""告警系统模块。

提供企业级告警通知功能：
- 慢请求、慢SQL、异常自动告警
- 累计触发和抑制机制
- 可扩展的通知渠道（内置飞书、Webhook）

快速开始：
    1. 配置环境变量
        ALERT_ENABLED=true
        ALERT_NOTIFIER_FEISHU_WEBHOOK=https://open.feishu.cn/...
    
    2. 在应用启动时初始化（FoundationApp 自动处理）
    
    3. 可选：添加规则文件 alert_rules.yaml

使用便捷函数发送自定义告警：
    from aury.boot.infrastructure.monitoring.alerting import emit_alert, AlertEventType
    
    await emit_alert(
        AlertEventType.CUSTOM,
        "自定义告警消息",
        severity=AlertSeverity.WARNING,
        my_data="xxx",
    )
"""

from .aggregator import AlertAggregator
from .events import AlertEvent, AlertEventType, AlertNotification, AlertSeverity
from .manager import AlertManager, emit_alert, emit_exception_alert
from .notifiers import AlertNotifier, FeishuNotifier, WebhookNotifier
from .rules import AlertRule, load_rules_from_dict

__all__ = [
    # 核心类
    "AlertAggregator",
    "AlertEvent",
    "AlertEventType",
    "AlertManager",
    "AlertNotification",
    "AlertRule",
    "AlertSeverity",
    # 通知器
    "AlertNotifier",
    "FeishuNotifier",
    "WebhookNotifier",
    # 便捷函数
    "emit_alert",
    "emit_exception_alert",
    "load_rules_from_dict",
]



"""错误响应模型（接口层）。

提供用于 HTTP API 响应的错误详情模型。
这是接口层的数据模型，用于序列化和传输。
"""

from __future__ import annotations

from typing import Any, ClassVar

from pydantic import BaseModel, ConfigDict, Field


class ErrorDetail(BaseModel):
    """错误详情模型（Pydantic）。
    
    支持两种错误类型：
    1. 字段错误：field不为空，表示特定字段的验证错误
    2. 通用错误：field为空，表示系统级或业务级错误
    
    使用示例:
        # 字段验证错误
        ErrorDetail(
            field="username",
            message="用户名格式不正确",
            code="INVALID_FORMAT",
            location="body"
        )
        
        # 通用业务错误
        ErrorDetail(
            message="库存不足",
            code="INSUFFICIENT_STOCK"
        )
        
        # 系统错误
        ErrorDetail(
            message="数据库连接失败",
            code="DB_CONNECTION_ERROR"
        )
    """
    
    message: str = Field(..., description="错误消息")
    code: str | None = Field(None, description="错误代码")
    field: str | None = Field(None, description="错误字段（仅字段验证错误）")
    location: str | None = Field(None, description="错误位置（如：body, query, path）")
    value: Any | None = Field(None, description="导致错误的值")
    
    @classmethod
    def field_error(
        cls,
        field: str,
        message: str,
        code: str | None = None,
        location: str = "body",
        value: Any | None = None,
    ) -> "ErrorDetail":
        """创建字段验证错误。
        
        Args:
            field: 字段名
            message: 错误消息
            code: 错误代码
            location: 错误位置
            value: 错误值
            
        Returns:
            ErrorDetail: 错误详情对象
        """
        return cls(
            message=message,
            code=code,
            field=field,
            location=location,
            value=value,
        )
    
    @classmethod
    def generic_error(
        cls,
        message: str,
        code: str | None = None,
    ) -> "ErrorDetail":
        """创建通用错误。
        
        Args:
            message: 错误消息
            code: 错误代码
            
        Returns:
            ErrorDetail: 错误详情对象
        """
        return cls(
            message=message,
            code=code,
        )
    
    model_config = ConfigDict(
        json_schema_extra={
            "examples": [
                {
                    "message": "用户名格式不正确",
                    "code": "INVALID_FORMAT",
                    "field": "username",
                    "location": "body",
                    "value": "user@123"
                },
                {
                    "message": "库存不足",
                    "code": "INSUFFICIENT_STOCK"
                }
            ]
        }
    )


__all__ = [
    "ErrorDetail",
]




"""日志上下文管理。

提供链路追踪 ID、服务上下文的管理。
"""

from __future__ import annotations

from contextvars import ContextVar
from enum import Enum
import uuid


class ServiceContext(str, Enum):
    """日志用服务上下文常量（避免跨层依赖）。"""
    API = "api"
    SCHEDULER = "scheduler"
    WORKER = "worker"


# 当前服务上下文（用于决定日志写入哪个文件）
_service_context: ContextVar[ServiceContext] = ContextVar("service_context", default=ServiceContext.API)

# 链路追踪 ID
_trace_id_var: ContextVar[str] = ContextVar("trace_id", default="")



def get_service_context() -> ServiceContext:
    """获取当前服务上下文。"""
    return _service_context.get()


def _to_service_context(ctx: ServiceContext | str) -> ServiceContext:
    """将输入标准化为 ServiceContext。"""
    if isinstance(ctx, ServiceContext):
        return ctx
    val = str(ctx).strip().lower()
    if val == "app":  # 兼容旧值
        val = ServiceContext.API.value
    try:
        return ServiceContext(val)
    except ValueError:
        return ServiceContext.API


def set_service_context(context: ServiceContext | str) -> None:
    """设置当前服务上下文。

    在调度器任务执行前调用 set_service_context("scheduler")，
    后续该任务中的所有日志都会写入 scheduler_xxx.log。

    Args:
        context: 服务类型（api/scheduler/worker，或兼容 "app"）
    """
    _service_context.set(_to_service_context(context))


def get_trace_id() -> str:
    """获取当前链路追踪ID。

    优先从 OpenTelemetry 获取（如果已启用），否则使用内置 trace_id。
    如果都没有设置，则生成一个新的随机 ID。
    """
    # 优先从 OTel 获取
    try:
        from opentelemetry import trace
        
        span = trace.get_current_span()
        if span and span.is_recording():
            otel_trace_id = span.get_span_context().trace_id
            if otel_trace_id:
                return format(otel_trace_id, "032x")
    except ImportError:
        pass
    except Exception:
        pass
    
    # 回退到内置实现
    trace_id = _trace_id_var.get()
    if not trace_id:
        trace_id = str(uuid.uuid4())
        _trace_id_var.set(trace_id)
    return trace_id


def set_trace_id(trace_id: str) -> None:
    """设置链路追踪ID。"""
    _trace_id_var.set(trace_id)


__all__ = [
    "ServiceContext",
    "get_service_context",
    "get_trace_id",
    "set_service_context",
    "set_trace_id",
]



"""应用层中间件模块。

仅包含 FastAPI HTTP 中间件。
事件中间件请参考 infrastructure.events。
"""

from .logging import RequestLoggingMiddleware, log_request

__all__ = [
    "RequestLoggingMiddleware",
    "log_request",
]













"""对象存储系统模块（统一出口）。

本包基于 aury-sdk-storage 提供的实现，对外暴露统一接口与管理器。
"""

# 从 SDK 直接导出核心类型
from aury.sdk.storage.storage import (
    COSStorage,  # 可选依赖，未安装 cos extras 时为 None
    IStorage,
    LocalStorage,
    S3Storage,  # 可选依赖，未安装 aws extras 时为 None
    StorageBackend,
    StorageConfig,
    StorageFile,
    StorageType,
    UploadResult,
)

# SDK 工厂（基于 StorageType 枚举）
from aury.sdk.storage.storage import StorageFactory as SDKStorageFactory

from .base import StorageManager
from .exceptions import StorageBackendError, StorageError, StorageNotFoundError

# Boot 工厂（注册机制）
from .factory import StorageFactory

__all__ = [
    # SDK 类型
    "COSStorage",
    "IStorage",
    "LocalStorage",
    "S3Storage",
    "StorageBackend",
    "StorageConfig",
    "StorageFile",
    "StorageType",
    "UploadResult",
    # 工厂
    "SDKStorageFactory",  # SDK 工厂（基于枚举类型）
    "StorageFactory",  # Boot 工厂（注册机制）
    # 异常
    "StorageBackendError",
    "StorageError",
    "StorageNotFoundError",
    # 管理器
    "StorageManager",
]




"""数据库相关异常定义。

Infrastructure 层异常，继承自 FoundationError。
"""

from __future__ import annotations

from aury.boot.common.exceptions import FoundationError


class DatabaseError(FoundationError):
    """数据库相关错误基类。
    
    所有数据库相关的异常都应该继承此类。
    """
    
    pass


class DatabaseConnectionError(DatabaseError):
    """数据库连接错误。"""
    
    pass


class DatabaseQueryError(DatabaseError):
    """数据库查询错误。"""
    
    pass


class DatabaseTransactionError(DatabaseError):
    """数据库事务错误。"""
    
    pass


__all__ = [
    "DatabaseConnectionError",
    "DatabaseError",
    "DatabaseQueryError",
    "DatabaseTransactionError",
]




"""Prometheus Metrics 模块。

提供基于 prometheus-fastapi-instrumentator 的 HTTP 指标采集。

使用方式：
    1. 安装依赖: pip install prometheus-fastapi-instrumentator
    2. 配置环境变量: PROMETHEUS_METRICS__ENABLED=true
    3. 访问 /metrics 端点查看指标

主要指标：
    - http_requests_total: HTTP 请求总数
    - http_request_duration_seconds: 请求延迟直方图
    - http_requests_in_progress: 进行中的请求数（活跃连接）
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from aury.boot.application.app.base import Plugin
from aury.boot.common.logging import logger

if TYPE_CHECKING:
    from aury.boot.application.app.base import FoundationApp
    from aury.boot.application.config import BaseConfig


class PrometheusMetricsPlugin(Plugin):
    """Prometheus Metrics 插件。
    
    在 app 创建后同步执行，负责：
    - 初始化 prometheus-fastapi-instrumentator
    - 配置指标采集选项
    - 暴露 /metrics 端点
    
    配置方式（环境变量）：
        PROMETHEUS_METRICS__ENABLED=true
        PROMETHEUS_METRICS__PATH=/metrics
        PROMETHEUS_METRICS__SHOULD_INSTRUMENT_REQUESTS_INPROGRESS=true
    """

    name = "metrics"
    enabled = True

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当 PROMETHEUS_METRICS__ENABLED=true 时启用。"""
        return self.enabled and config.prometheus_metrics.enabled

    def install(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化 Prometheus Metrics 并暴露端点。"""
        try:
            from prometheus_fastapi_instrumentator import Instrumentator
        except ImportError:
            logger.warning(
                "prometheus-fastapi-instrumentator 未安装，跳过 Metrics 初始化。"
                "请运行: pip install prometheus-fastapi-instrumentator"
            )
            return

        try:
            metrics_config = config.prometheus_metrics
            
            # 创建 Instrumentator 实例
            instrumentator = Instrumentator(
                should_group_status_codes=metrics_config.should_group_status_codes,
                should_ignore_untemplated=metrics_config.should_ignore_untemplated,
                should_group_untemplated=metrics_config.should_group_untemplated,
                should_round_latency_decimals=metrics_config.should_round_latency_decimals,
                should_respect_env_var=metrics_config.should_respect_env_var,
                should_instrument_requests_inprogress=metrics_config.should_instrument_requests_inprogress,
                excluded_handlers=metrics_config.excluded_handlers,
            )
            
            # Instrument app
            instrumentator.instrument(app)
            
            # 暴露 /metrics 端点
            instrumentator.expose(
                app,
                endpoint=metrics_config.path,
                include_in_schema=metrics_config.include_in_schema,
            )
            
            # 保存到 app.state 以便后续访问
            app.state.metrics_instrumentator = instrumentator
            
            logger.info(f"Prometheus Metrics 已启用，端点: {metrics_config.path}")
            
        except Exception as e:
            logger.warning(f"Prometheus Metrics 初始化失败（非关键）: {e}")

    def uninstall(self, app: FoundationApp) -> None:
        """清理 Metrics 资源。"""
        if hasattr(app.state, "metrics_instrumentator"):
            delattr(app.state, "metrics_instrumentator")
            logger.debug("Prometheus Metrics 已清理")


__all__ = ["PrometheusMetricsPlugin"]



"""告警通知器模块。

提供内置的通知器实现：
- FeishuNotifier: 飞书机器人
- WebhookNotifier: 通用 Webhook
"""

from .base import AlertNotifier
from .feishu import FeishuNotifier
from .webhook import WebhookNotifier

__all__ = [
    "AlertNotifier",
    "FeishuNotifier",
    "WebhookNotifier",
]



"""异步任务管理器模块。

提供统一的任务队列接口，支持条件注册以避免 API 模式下重复注册。
"""

from .constants import TaskQueueName, TaskRunMode
from .exceptions import (
    TaskError,
    TaskExecutionError,
    TaskQueueError,
)
from .manager import TaskManager, TaskProxy, conditional_task

__all__ = [
    "TaskError",
    "TaskExecutionError",
    "TaskManager",
    "TaskProxy",
    "TaskQueueError",
    "TaskQueueName",
    "TaskRunMode",
    "conditional_task",
]




"""项目脚手架初始化命令。

类似 Vue CLI 的交互式项目初始化。

前置条件：
    1. mkdir my-service && cd my-service
    2. uv init . --bare --name my_service
    3. uv venv --python 3.13
    4. uv add "aury-boot[recommended]"

初始化脚手架：
    aury init                    # 交互式初始化
    aury init -r                 # 推荐配置快速初始化
    aury init my_package         # 指定顶层包名
"""

from __future__ import annotations

from enum import Enum
from pathlib import Path
import sys

from rich.console import Console
from rich.panel import Panel
from rich.prompt import Confirm, IntPrompt, Prompt
from rich.tree import Tree
import typer

from .config import ProjectConfig, save_project_config

console = Console()

# 最低 Python 版本要求
MIN_PYTHON_VERSION = (3, 13)


# ============================================================
# 枚举定义
# ============================================================


class ServiceMode(str, Enum):
    """服务运行模式。"""

    API = "api"
    API_SCHEDULER = "api+scheduler"
    FULL = "full"


class CacheType(str, Enum):
    """缓存类型。"""

    MEMORY = "memory"
    REDIS = "redis"


class DatabaseType(str, Enum):
    """数据库类型。"""

    POSTGRESQL = "postgresql"
    MYSQL = "mysql"
    SQLITE = "sqlite"


# ============================================================
# 依赖配置
# ============================================================


# 模板目录
TEMPLATES_DIR = Path(__file__).parent / "templates" / "project"
MODULE_TEMPLATES_DIR = TEMPLATES_DIR / "modules"

# 需要创建的目录结构（包内）
DIRECTORIES = [
    "api",
    "services",
    "models",
    "repositories",
    "schemas",
    "exceptions",  # 自定义异常
    "tasks",       # 异步任务（Dramatiq）
    "schedules",   # 定时任务（Scheduler）
]

# Ruff 配置
RUFF_CONFIG = '''
[tool.ruff]
target-version = "py313"
line-length = 120
indent-width = 4
exclude = [
    ".git",
    ".venv",
    "__pycache__",
    "*.pyc",
    ".mypy_cache",
    ".pytest_cache",
    ".ruff_cache",
    "build",
    "dist",
    "*.egg-info",
]

[tool.ruff.lint]
select = [
    "E",      # pycodestyle errors
    "W",      # pycodestyle warnings
    "F",      # Pyflakes
    "I",      # isort
    "N",      # pep8-naming
    "UP",     # pyupgrade
    "B",      # flake8-bugbear
    "C4",     # flake8-comprehensions
    "SIM",    # flake8-simplify
    "T20",    # flake8-print
    "RUF",    # Ruff-specific rules
]
ignore = [
    "E501",   # 行长度
    "B008",   # 函数调用中的默认参数
    "B006",   # 可变默认参数
    "T201",   # print 语句
    "RUF001", # 中文标点
    "RUF002", # 中文标点
    "RUF003", # 中文标点
]

[tool.ruff.lint.isort]
known-first-party = ["app", "api", "models", "services", "repositories", "schemas"]
force-sort-within-sections = true

[tool.pytest.ini_options]
asyncio_mode = "auto"
testpaths = ["tests"]
'''

# 开发依赖配置（单独处理，确保总是添加）
DEV_DEPS_CONFIG = '''
[dependency-groups]
dev = [
    "pytest>=9.0.1",
    "pytest-asyncio>=1.3.0",
    "pytest-cov>=7.0.0",
    "ruff>=0.14.0",
    "mypy>=1.19.0",
    "watchfiles>=0.24.0",
]
'''


# 模板文件映射（模板名 -> .tpl 文件名）
TEMPLATE_FILE_MAP = {
    "main.py": "main.py.tpl",
    "config.py": "config.py.tpl",
    ".env.example": "env_templates",  # 特殊处理：拼接 env_templates/ 目录下的所有 .tpl 文件
    ".gitignore": "gitignore.tpl",
    "README.md": "README.md.tpl",
    "AGENTS.md": "AGENTS.md.tpl",
    "conftest.py": "conftest.py.tpl",
    "admin_console/__init__.py": "admin_console_init.py.tpl",
    "alert_rules.example.yaml": "alert_rules.example.yaml.tpl",
}

# env 模板拼接顺序
ENV_TEMPLATE_ORDER = [
    "_header.tpl",
    "service.tpl",
    "database.tpl",
    "cache.tpl",
    "log.tpl",
    "admin.tpl",
    "scheduler.tpl",
    "messaging.tpl",
    "storage.tpl",
    "monitoring.tpl",
    "third_party.tpl",
    "rpc.tpl",
]

# 模块 __init__.py 模板映射
MODULE_TEMPLATE_MAP = {
    "api": "api.py.tpl",
    "tasks": "tasks.py.tpl",
    "schedules": "schedules.py.tpl",
    "exceptions": "exceptions.py.tpl",
}


def _read_env_template() -> str:
    """读取并拼接 env_templates/ 目录下的所有模板文件。"""
    env_dir = TEMPLATES_DIR / "env_templates"
    parts = []
    
    for tpl_name in ENV_TEMPLATE_ORDER:
        tpl_path = env_dir / tpl_name
        if tpl_path.exists():
            parts.append(tpl_path.read_text(encoding="utf-8"))
    
    return "\n".join(parts)


def _read_template(name: str) -> str:
    """读取模板文件。"""
    # 特殊处理 .env.example：拼接 env_templates/ 目录
    if name == ".env.example":
        return _read_env_template()
    
    # 先尝试从映射中查找 .tpl 文件
    tpl_name = TEMPLATE_FILE_MAP.get(name)
    if tpl_name:
        template_path = TEMPLATES_DIR / tpl_name
        if template_path.exists():
            return template_path.read_text(encoding="utf-8")
    
    # 尝试直接读取
    template_path = TEMPLATES_DIR / name
    if template_path.exists():
        return template_path.read_text(encoding="utf-8")
    
    # 如果模板文件不存在，抛出错误
    raise FileNotFoundError(f"模板文件不存在: {name} (查找路径: {TEMPLATES_DIR})")


def _read_module_template(module_name: str) -> str:
    """读取模块 __init__.py 模板。"""
    tpl_name = MODULE_TEMPLATE_MAP.get(module_name)
    if tpl_name:
        template_path = MODULE_TEMPLATES_DIR / tpl_name
        if template_path.exists():
            return template_path.read_text(encoding="utf-8")
    # 如果模板文件不存在，抛出错误
    raise FileNotFoundError(f"模块模板文件不存在: {module_name} (查找路径: {MODULE_TEMPLATES_DIR})")


# ============================================================
# Admin Console 模块初始化（可复用）
# ============================================================


def init_admin_console_module(
    base_path: Path,
    code_root: Path,
    import_prefix: str,
    *,
    force: bool = False,
    enable_env: bool = True,
) -> dict[str, bool]:
    """初始化 Admin Console 模块到现有项目。

    用于 `aury init`（新项目）和 `aury add admin-console`（已有项目）复用。

    Args:
        base_path: 项目根目录
        code_root: 代码根目录（包根或平铺）
        import_prefix: 导入前缀（如 "app."）
        force: 强制覆盖已有 admin_console/__init__.py
        enable_env: 是否尝试在 .env.example 中开启 ADMIN_* 配置

    Returns:
        dict 包含操作结果:
        - file_created: 文件是否创建成功
        - file_existed: 文件是否已存在（且未覆盖）
        - env_updated: 是否更新了 .env.example
    """
    result = {"file_created": False, "file_existed": False, "env_updated": False}

    # 1) 创建 admin_console/ 包目录和 __init__.py
    admin_pkg = code_root / "admin_console"
    dest = admin_pkg / "__init__.py"
    content = _read_template("admin_console/__init__.py")
    content = content.replace("{import_prefix}", import_prefix)

    if dest.exists() and not force:
        result["file_existed"] = True
    else:
        admin_pkg.mkdir(parents=True, exist_ok=True)
        dest.write_text(content, encoding="utf-8")
        result["file_created"] = True

    # 2) 尝试在 .env.example 中开启 ADMIN__* 配置
    if enable_env:
        env_example = base_path / ".env.example"
        if env_example.exists():
            try:
                s = env_example.read_text(encoding="utf-8")
                s2 = (
                    s.replace("# ADMIN__ENABLED=false", "ADMIN__ENABLED=true")
                     .replace("# ADMIN__PATH=/api/admin-console", "ADMIN__PATH=/api/admin-console")
                     .replace("# ADMIN__AUTH__MODE=basic", "ADMIN__AUTH__MODE=basic")
                     .replace(
                        "# ADMIN__AUTH__SECRET_KEY=CHANGE_ME_TO_A_RANDOM_SECRET",
                        "ADMIN__AUTH__SECRET_KEY=CHANGE_ME_TO_A_RANDOM_SECRET",
                    )
                     .replace("# ADMIN__AUTH__BASIC_USERNAME=admin", "ADMIN__AUTH__BASIC_USERNAME=admin")
                     .replace("# ADMIN__AUTH__BASIC_PASSWORD=change_me", "ADMIN__AUTH__BASIC_PASSWORD=change_me")
                )
                if s2 != s:
                    env_example.write_text(s2, encoding="utf-8")
                    result["env_updated"] = True
            except Exception:
                pass  # 静默失败

    return result


def _create_directory_structure(base_path: Path, package_name: str | None = None) -> list[str]:
    """创建目录结构。

    Args:
        base_path: 项目根目录
        package_name: 顶层包名，None 表示平铺结构

    Returns:
        创建的目录列表
    """
    created = []

    # 确定代码根目录
    if package_name:
        code_root = base_path / package_name
        # 创建顶层包目录
        if not code_root.exists():
            code_root.mkdir(parents=True, exist_ok=True)
            created.append(package_name)
            # 创建顶层 __init__.py
            (code_root / "__init__.py").write_text(
                f'"""顶层包 {package_name}。"""\n', encoding="utf-8"
            )
    else:
        code_root = base_path

    for dir_path in DIRECTORIES:
        full_path = code_root / dir_path
        if not full_path.exists():
            full_path.mkdir(parents=True, exist_ok=True)
            rel_path = f"{package_name}/{dir_path}" if package_name else dir_path
            created.append(rel_path)
            # 创建 __init__.py
            init_file = full_path / "__init__.py"
            if not init_file.exists():
                # 尝试从外部模板读取
                if dir_path in MODULE_TEMPLATE_MAP:
                    init_file.write_text(_read_module_template(dir_path), encoding="utf-8")
                else:
                    # 普通目录使用简单的 __init__.py
                    init_file.write_text('"""模块初始化。"""\n', encoding="utf-8")
    return created


def _create_file_if_not_exists(path: Path, content: str) -> bool:
    """创建文件（如果不存在）。"""
    if path.exists():
        return False
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(content, encoding="utf-8")
    return True


def _append_ruff_config(base_path: Path) -> bool:
    """追加 ruff 配置到 pyproject.toml。"""
    pyproject_path = base_path / "pyproject.toml"
    if not pyproject_path.exists():
        return False
    
    content = pyproject_path.read_text(encoding="utf-8")
    
    # 检查是否已有 ruff 配置
    if "[tool.ruff]" in content:
        return False
    
    content += RUFF_CONFIG
    pyproject_path.write_text(content, encoding="utf-8")
    return True


def _append_dev_deps_config(base_path: Path) -> bool:
    """追加开发依赖配置到 pyproject.toml。"""
    pyproject_path = base_path / "pyproject.toml"
    if not pyproject_path.exists():
        return False
    
    content = pyproject_path.read_text(encoding="utf-8")
    
    # 检查是否已有 dependency-groups 配置
    if "[dependency-groups]" in content:
        return False
    
    content += DEV_DEPS_CONFIG
    pyproject_path.write_text(content, encoding="utf-8")
    return True


def _to_snake_case(name: str) -> str:
    """转换为 snake_case。"""
    import re
    # 处理 PascalCase 和 camelCase
    s1 = re.sub(r"(.)([A-Z][a-z]+)", r"\1_\2", name)
    return re.sub(r"([a-z0-9])([A-Z])", r"\1_\2", s1).lower().replace("-", "_")


def _init_migrations(base_path: Path, package_name: str | None = None) -> bool:
    """初始化数据库迁移目录和配置。

    Args:
        base_path: 项目根目录
        package_name: 顶层包名，None 表示平铺结构
    
    直接调用 MigrationManager 的创建逻辑，保证单一数据源。
    """
    from aury.boot.application.config.settings import MigrationSettings
    from aury.boot.application.migrations.setup import ensure_migration_setup
    
    migration_config = MigrationSettings()
    migrations_dir = base_path / migration_config.script_location
    
    if migrations_dir.exists():
        return False
    
    # 调用统一的创建函数
    ensure_migration_setup(
        base_path=base_path,
        config_path=migration_config.config_path,
        script_location=migration_config.script_location,
        model_modules=migration_config.model_modules,
    )
    
    return True


# ============================================================
# 环境检查函数
# ============================================================


def _check_python_version() -> bool:
    """检查 Python 版本是否满足最低要求。"""
    return sys.version_info >= MIN_PYTHON_VERSION


# ============================================================
# 交互式配置收集
# ============================================================


def _collect_interactive_config() -> dict:
    """交互式收集项目配置（Vue CLI 风格）。"""
    config = {}

    # 默认包名使用 app（更符合习惯，也更利于 AI/模板稳定生成）
    # 如需平铺结构请输入 "."
    default_pkg = "app"

    console.print(Panel.fit(
        "[bold cyan]🎯 Aury Boot[/bold cyan]\n"
        "[dim]交互式项目初始化[/dim]",
        border_style="cyan",
    ))
    console.print()

    # 1. 项目结构（输入包名或留空）
    console.print("[bold]📦 项目结构[/bold]")
    console.print("  [dim]输入包名使用顶层包结构，. 则使用平铺结构[/dim]")
    package_input = Prompt.ask(
        "包名",
        default=default_pkg,
    )
    # "." 表示平铺结构
    if package_input.strip() == ".":
        config["package_name"] = None
    else:
        config["package_name"] = package_input.strip() or None

    # 2. 数据库类型
    console.print()
    console.print("[bold]🗄️  数据库[/bold]")
    console.print("  [dim]1. PostgreSQL (推荐)[/dim]")
    console.print("  [dim]2. MySQL[/dim]")
    console.print("  [dim]3. SQLite (开发用)[/dim]")
    db_choice = IntPrompt.ask(
        "选择数据库",
        default=1,
        choices=["1", "2", "3"],
    )
    config["database"] = {
        1: "postgresql",
        2: "mysql",
        3: "sqlite",
    }[db_choice]

    # 3. 缓存类型
    console.print()
    console.print("[bold]📦 缓存[/bold]")
    console.print("  [dim]1. Redis (推荐)[/dim]")
    console.print("  [dim]2. 内存缓存 (开发用)[/dim]")
    cache_choice = IntPrompt.ask(
        "选择缓存类型",
        default=1,
        choices=["1", "2"],
    )
    config["cache"] = {
        1: "redis",
        2: "memory",
    }[cache_choice]

    # 4. 服务模式（决定推荐安装的依赖包）
    console.print()
    console.print("[bold]⚙️  服务模式[/bold] [dim](决定推荐安装的依赖)[/dim]")
    console.print("  [dim]1. api           - 纯 API 服务[/dim]")
    console.print("  [dim]2. api+scheduler - API + 定时任务 (APScheduler)[/dim]")
    console.print("  [dim]3. full          - API + 定时任务 + 异步任务队列 (Dramatiq)[/dim]")
    mode_choice = IntPrompt.ask(
        "选择服务模式",
        default=2,
        choices=["1", "2", "3"],
    )
    config["service_mode"] = {
        1: "api",
        2: "api+scheduler",
        3: "full",
    }[mode_choice]

    # 5. 可选功能
    console.print()
    console.print("[bold]📦 可选功能[/bold]")
    features = []

    # 管理后台（Admin Console）
    config["with_admin_console"] = Confirm.ask(
        "  启用管理后台 Admin Console (SQLAdmin)",
        default=True,
    )

    if Confirm.ask("  启用对象存储 (S3/本地)", default=True):
        features.append("storage")

    if Confirm.ask("  启用事件总线", default=False):
        features.append("events")

    if Confirm.ask("  启用国际化 (i18n)", default=False):
        features.append("i18n")

    config["features"] = features

    # 5.5 监控配置
    console.print()
    console.print("[bold]📊 监控配置[/bold]")
    config["with_otel"] = Confirm.ask(
        "  启用 OpenTelemetry 链路追踪",
        default=True,
    )
    config["with_profiling"] = Confirm.ask(
        "  启用 Profiling (火焰图/阻塞检测)",
        default=False,
    )

    # 6. 开发工具
    console.print()
    config["with_dev"] = Confirm.ask(
        "[bold]🛠️  安装开发工具[/bold] (pytest, ruff, mypy)",
        default=True,
    )

    # 7. Docker 配置
    console.print()
    config["with_docker"] = Confirm.ask(
        "[bold]🐳 生成 Docker 配置[/bold]",
        default=True,
    )

    return config


def _build_dependency_list(config: dict) -> list[str]:
    """根据配置构建依赖列表。"""
    extras = set()

    # 数据库
    db = config.get("database", "postgresql")
    if db == "postgresql":
        extras.add("postgresql")
    elif db == "mysql":
        extras.add("mysql")

    # 缓存
    if config.get("cache") == "redis":
        extras.add("redis")

    # 服务模式
    mode = config.get("service_mode", "api")
    if mode in ("api+scheduler", "full"):
        extras.add("scheduler")
    if mode == "full":
        extras.add("tasks")

    # 可选功能
    for feature in config.get("features", []):
        extras.add(feature)

    # 管理后台（可选扩展）
    if config.get("with_admin_console", True):
        extras.add("admin")

    # 监控
    if config.get("with_otel", True):
        extras.add("otel")
    if config.get("with_profiling", False):
        extras.add("profiling")

    # 开发工具
    if config.get("with_dev"):
        extras.add("dev")

    # 构建依赖字符串
    if extras:
        extras_str = ",".join(sorted(extras))
        return [f"aury-boot[{extras_str}]"]
    return ["aury-boot"]


def _show_config_summary(config: dict) -> None:
    """显示配置摘要。"""
    console.print()
    console.print(Panel.fit(
        "[bold]📋 配置摘要[/bold]",
        border_style="blue",
    ))

    items = [
        ("项目名称", config.get("project_name", Path.cwd().name)),
        ("包结构", config.get("package_name") or "平铺结构"),
        ("数据库", config.get("database", "postgresql")),
        ("缓存", config.get("cache", "memory")),
        ("服务模式", config.get("service_mode", "api")),
        ("管理后台", "是" if config.get("with_admin_console", True) else "否"),
        ("可选功能", ", ".join(config.get("features", [])) or "无"),
        ("OpenTelemetry", "是" if config.get("with_otel", True) else "否"),
        ("Profiling", "是" if config.get("with_profiling", False) else "否"),
        ("开发工具", "是" if config.get("with_dev") else "否"),
        ("Docker", "是" if config.get("with_docker") else "否"),
    ]

    for label, value in items:
        console.print(f"  [bold]{label}:[/bold] {value}")

    # 显示依赖
    deps = _build_dependency_list(config)
    console.print(f"  [bold]依赖:[/bold] {deps[0]}")


# ============================================================
# 主命令
# ============================================================


def init(
    package_name: str = typer.Argument(
        None,
        help="顶层包名（默认 app）。如需平铺结构请输入 '.'",
    ),
    no_interactive: bool = typer.Option(
        False,
        "--no-interactive",
        "-y",
        help="跳过交互，使用默认配置",
    ),
    force: bool = typer.Option(
        False,
        "--force",
        "-f",
        help="强制覆盖已存在的文件（包括 main.py）",
    ),
    with_docker: bool = typer.Option(
        False,
        "--docker",
        help="同时生成 Docker 配置文件",
    ),
) -> None:
    """初始化 Aury 项目脚手架。

    前置条件（先执行以下命令）：
        mkdir my-service && cd my-service
        uv init . --name my_service --no-package --python 3.13
        uv add "aury-boot[recommended]"

    示例：
        aury init                    # 交互式模式（默认）
        aury init -y                 # 跳过交互，使用默认配置
        aury init my_package         # 顶层包结构
        aury init --docker           # 包含 Docker 配置
        aury init -f                 # 强制覆盖
    """
    base_path = Path.cwd()

    # 检查 pyproject.toml
    if not (base_path / "pyproject.toml").exists():
        console.print("[red]❌ 未找到 pyproject.toml[/red]")
        console.print()
        console.print("[bold]请先执行以下命令：[/bold]")
        console.print("  [cyan]uv init . --name <project_name> --no-package --python 3.13[/cyan]")
        console.print('  [cyan]uv add "aury-boot[recommended]"[/cyan]')
        raise typer.Exit(1)

    # 获取项目名称
    project_name = base_path.name
    project_name_snake = _to_snake_case(project_name)

    # 交互式模式（默认）
    if not no_interactive:
        config = _collect_interactive_config()
        config["project_name"] = project_name  # 使用当前目录名
        package_name_snake = _to_snake_case(config.get("package_name")) if config.get("package_name") else None
        with_docker = config.get("with_docker", False)
        with_admin_console = config.get("with_admin_console", True)

        # 显示配置摘要并确认
        _show_config_summary(config)
        console.print()
        if not Confirm.ask("确认初始化项目", default=True):
            console.print("[yellow]已取消[/yellow]")
            raise typer.Exit(0)

        # 显示推荐的依赖安装命令
        deps = _build_dependency_list(config)
        console.print()
        console.print("[bold]📦 推荐安装的依赖：[/bold]")
        console.print(f"  [cyan]uv add \"{deps[0]}\"[/cyan]")
        console.print()
    else:
        # 非交互模式：默认禁用 Admin Console
        with_admin_console = False
        if package_name == ".":
            package_name_snake = None
        elif package_name:
            package_name_snake = _to_snake_case(package_name)
        else:
            package_name_snake = "app"

    # 显示标题
    title = project_name
    if package_name_snake:
        title += f" (包: {package_name_snake})"
    console.print(Panel.fit(
        f"[bold cyan]🚀 初始化 Aury 项目: {title}[/bold cyan]",
        border_style="cyan",
    ))

    # 1. 创建目录结构
    console.print("\n[bold]📁 创建目录结构...[/bold]")
    _create_directory_structure(base_path, package_name_snake)

    # 确定代码目录
    code_root = base_path / package_name_snake if package_name_snake else base_path

    # 2. 生成文件
    console.print("\n[bold]📝 生成文件...[/bold]")

    # main.py 总是覆盖（因为 uv init 会创建默认的）
    # main.py 始终放在根目录，作为入口文件
    files_to_create = [
        (base_path / "main.py", "main.py", True),  # 总是覆盖，放在根目录
        (code_root / "config.py", "config.py", False),
        (base_path / ".env.example", ".env.example", False),
        # 管理后台默认模块（可选）—— 现在是包目录
        (code_root / "admin_console" / "__init__.py", "admin_console/__init__.py", False),
        (base_path / "tests" / "conftest.py", "conftest.py", False),  # tests 放在项目根目录
        (base_path / "README.md", "README.md", True),  # 覆盖 uv init 创建的默认 README
        (base_path / "AGENTS.md", "AGENTS.md", False),  # AI 编程助手上下文
        (base_path / ".gitignore", ".gitignore", False),  # Git 忽略文件
    ]

    import_prefix = f"{package_name_snake}." if package_name_snake else ""
    template_vars = {
        "project_name": project_name,
        "project_name_snake": project_name_snake,
        "import_prefix": import_prefix,
        "package_name": package_name_snake or "",
    }

    for full_path, template_name, always_overwrite in files_to_create:
        # 若禁用管理后台，则跳过生成 admin_console/
        if template_name == "admin_console/__init__.py" and not with_admin_console:
            continue

        rel_path = full_path.relative_to(base_path)
        should_write = always_overwrite or force or not full_path.exists()

        if not should_write:
            console.print(f"  [dim]⏭️  {rel_path} 已存在，跳过[/dim]")
            continue

        content = _read_template(template_name)
        # 临时替换代码块中的字典字面量，避免 str.format() 解析
        import re
        dict_placeholders = {}
        placeholder_counter = [0]  # 使用列表以便在嵌套函数中修改
        
        def protect_dict(match, *, _dict_placeholders=dict_placeholders, _placeholder_counter=placeholder_counter):
            """保护字典字面量，用占位符替换"""
            placeholder = f"__DICT_PLACEHOLDER_{_placeholder_counter[0]}__"
            _dict_placeholders[placeholder] = match.group(0)
            _placeholder_counter[0] += 1
            return placeholder
        
        def process_code_block(match, *, _protect_dict=protect_dict):
            """处理代码块，保护其中的字典字面量"""
            code_content = match.group(1)
            protected_code = re.sub(r'\{"[^"]+":\s*[^}]+\}', _protect_dict, code_content, flags=re.DOTALL)
            return '```python' + protected_code + '```'
        
        # 在代码块中保护字典字面量（匹配 {"key": value} 格式）
        content = re.sub(r'```python(.*?)```', process_code_block, content, flags=re.DOTALL)
        
        # 格式化模板（替换 {project_name} 等占位符）
        # 合并 template_vars 和 dict_placeholders，防止 KeyError
        format_vars = {**template_vars, **dict_placeholders}
        content = content.format(**format_vars)

        # 若启用管理后台，默认在 .env.example 中打开 ADMIN_ENABLED，并给出基础示例
        if template_name == ".env.example" and with_admin_console:
            content = content.replace("# ADMIN_ENABLED=false", "ADMIN_ENABLED=true")
            content = content.replace("# ADMIN_PATH=/api/admin-console", "ADMIN_PATH=/api/admin-console")
            content = content.replace("# ADMIN_AUTH_MODE=basic", "ADMIN_AUTH_MODE=basic")
            content = content.replace(
                "# ADMIN_AUTH_SECRET_KEY=CHANGE_ME_TO_A_RANDOM_SECRET",
                "ADMIN_AUTH_SECRET_KEY=CHANGE_ME_TO_A_RANDOM_SECRET",
            )
            content = content.replace("# ADMIN_AUTH_BASIC_USERNAME=admin", "ADMIN_AUTH_BASIC_USERNAME=admin")
            content = content.replace("# ADMIN_AUTH_BASIC_PASSWORD=change_me", "ADMIN_AUTH_BASIC_PASSWORD=change_me")
        
        # 恢复字典字面量
        for placeholder, original in dict_placeholders.items():
            content = content.replace(placeholder, original)
        full_path.parent.mkdir(parents=True, exist_ok=True)
        full_path.write_text(content, encoding="utf-8")
        console.print(f"  [green]✅ {rel_path}[/green]")

    # 3. 配置 pyproject.toml
    console.print("\n[bold]⚙️  配置 pyproject.toml...[/bold]")

    # 保存 [tool.aury] 配置（包含 package 与 app）
    proj_config = ProjectConfig(package=package_name_snake, app="main:app")
    if save_project_config(proj_config, base_path):
        if package_name_snake:
            console.print(f"  [green]✅ 已保存包配置: [tool.aury] package = \"{package_name_snake}\"[/green]")
        console.print("  [green]✅ 已保存入口配置: [tool.aury] app = \"main:app\"[/green]")

    if _append_ruff_config(base_path):
        console.print("  [green]✅ 已添加 ruff 和 pytest 配置[/green]")
    
    if _append_dev_deps_config(base_path):
        console.print("  [green]✅ 已添加 [dependency-groups] dev 配置[/green]")

    # 4. 初始化数据库迁移
    console.print("\n[bold]📦 初始化数据库迁移...[/bold]")
    if _init_migrations(base_path, package_name_snake):
        console.print("  [green]✅ 已创建 migrations/ 目录和配置[/green]")
    else:
        console.print("  [dim]ℹ️  migrations/ 目录已存在，跳过[/dim]")

    # 5. 生成开发文档 (aury_docs/) - 复用 docs.generate_aury_docs
    console.print("\n[bold]📚 生成开发文档...[/bold]")
    from .docs import generate_aury_docs

    docs_count = generate_aury_docs(
        project_dir=base_path,
        context=template_vars,
        force=force,
        dry_run=False,
        quiet=True,
    )
    console.print(f"  [green]✅ 已生成 {docs_count} 个文档到 aury_docs/[/green]")

    # 6. 生成 Docker 配置
    if with_docker:
        console.print("\n[bold]🐳 生成 Docker 配置...[/bold]")
        from .docker import docker_init
        docker_init(force=force)

    # 7. 显示结果
    console.print("\n")

    tree = Tree(f"[bold cyan]{project_name}/[/bold cyan]")
    tree.add("[dim].env.example[/dim]")
    tree.add("[dim].gitignore[/dim]")
    tree.add("[dim]alembic.ini[/dim]")
    tree.add("[dim]pyproject.toml[/dim]")
    tree.add("[dim]README.md[/dim]")
    tree.add("[dim]AGENTS.md[/dim]")
    tree.add("[blue]aury_docs/[/blue]")
    if with_docker:
        tree.add("[dim]Dockerfile[/dim]")
        tree.add("[dim]docker-compose.yml[/dim]")
        tree.add("[dim].dockerignore[/dim]")

    # tests 目录始终在项目根目录
    tests_branch = tree.add("[blue]tests/[/blue]")
    tests_branch.add("[dim]conftest.py[/dim]")
    
    # main.py 始终在根目录
    tree.add("[green]main.py[/green]")
    
    if package_name_snake:
        pkg_branch = tree.add(f"[bold blue]{package_name_snake}/[/bold blue]")
        pkg_branch.add("[green]config.py[/green]")
        pkg_branch.add("[blue]api/[/blue]")
        pkg_branch.add("[blue]services/[/blue]")
        pkg_branch.add("[blue]models/[/blue]")
        pkg_branch.add("[blue]repositories/[/blue]")
        pkg_branch.add("[blue]schemas/[/blue]")
        pkg_branch.add("[blue]exceptions/[/blue]")
        pkg_branch.add("[blue]tasks/[/blue]")
        pkg_branch.add("[blue]schedules/[/blue]")
    else:
        tree.add("[green]config.py[/green]")
        tree.add("[blue]api/[/blue]")
        tree.add("[blue]services/[/blue]")
        tree.add("[blue]models/[/blue]")
        tree.add("[blue]repositories/[/blue]")
        tree.add("[blue]schemas/[/blue]")
        tree.add("[blue]exceptions/[/blue]")
        tree.add("[blue]tasks/[/blue]")
        tree.add("[blue]schedules/[/blue]")

    migrations_branch = tree.add("[blue]migrations/[/blue]")
    migrations_branch.add("[dim]env.py[/dim]")
    migrations_branch.add("[dim]versions/[/dim]")

    console.print(tree)

    # 下一步提示
    console.print("\n[bold green]✨ 项目初始化完成！[/bold green]\n")
    console.print("[bold]下一步：[/bold]")
    console.print("  1. 安装开发依赖：")
    console.print("     [cyan]uv sync --group dev[/cyan]")
    console.print("     [dim]dev 组包含: pytest, pytest-asyncio, pytest-cov, ruff, mypy, watchfiles（用于稳定热重载）[/dim]")
    console.print("  2. 复制并编辑环境变量：")
    console.print("     [cyan]cp .env.example .env[/cyan]")
    console.print("     [dim]# 编辑 .env 配置数据库连接等[/dim]")
    console.print("  3. 启动开发服务器：")
    console.print("     [cyan]aury server dev[/cyan]")
    console.print("  4. 访问 API 文档：")
    console.print("     [cyan]http://127.0.0.1:8000/docs[/cyan]")
    console.print()
    console.print("[bold]包管理（按需安装可选模块）：[/bold]")
    console.print("  [cyan]aury pkg list[/cyan]               # 查看所有可用模块")
    console.print("  [cyan]aury pkg preset[/cyan]             # 查看预设配置")
    console.print("  [cyan]aury pkg install postgres redis[/cyan]  # 安装指定模块")
    console.print("  [cyan]aury pkg install --preset api[/cyan]    # 按预设安装")
    console.print()
    console.print("[bold]常用命令：[/bold]")
    console.print("  [cyan]aury generate crud user -i[/cyan]  # 生成 CRUD（交互式）")
    console.print("  [cyan]aury generate model user -i[/cyan] # 生成模型（交互式）")
    console.print("  [cyan]aury migrate make -m \"xxx\"[/cyan] # 创建迁移")
    console.print("  [cyan]aury migrate up[/cyan]             # 执行迁移")
    console.print("  [cyan]aury server prod[/cyan]           # 生产模式")
    console.print()
    console.print("[dim]💡 使用 -i 参数可交互式配置字段、类型、约束等[/dim]")
    console.print()
    console.print("[dim]详细文档: https://github.com/AuriMyth/aury-boot[/dim]")


__all__ = ["init", "init_admin_console_module"]



"""飞书通知器。

通过飞书机器人 Webhook 发送告警通知。
"""

from __future__ import annotations

import base64
import hashlib
import hmac
import time
from typing import TYPE_CHECKING, Any

from aury.boot.common.logging import logger

from .base import AlertNotifier

if TYPE_CHECKING:
    from ..events import AlertNotification


class FeishuNotifier(AlertNotifier):
    """飞书机器人通知器。
    
    通过飞书自定义机器人 Webhook 发送告警。
    支持签名校验（可选）。
    
    环境变量配置示例：
        ALERT_NOTIFIER_FEISHU_TYPE=feishu
        ALERT_NOTIFIER_FEISHU_WEBHOOK=https://open.feishu.cn/open-apis/bot/v2/hook/xxx
        ALERT_NOTIFIER_FEISHU_SECRET=xxx  # 可选，签名密钥
    """
    
    def __init__(self, webhook: str, secret: str | None = None) -> None:
        """初始化飞书通知器。
        
        Args:
            webhook: 飞书机器人 Webhook URL
            secret: 签名密钥（可选）
        """
        self.webhook = webhook
        self.secret = secret
    
    @classmethod
    def from_config(cls, config: dict) -> "FeishuNotifier":
        """从配置创建实例。"""
        webhook = config.get("webhook")
        if not webhook:
            raise ValueError("飞书通知器配置缺少 webhook")
        return cls(webhook=webhook, secret=config.get("secret"))
    
    def _generate_sign(self, timestamp: int) -> str:
        """生成签名。
        
        飞书签名算法：
        sign = base64(hmac-sha256(timestamp + "\\n" + secret, secret))
        """
        if not self.secret:
            return ""
        
        string_to_sign = f"{timestamp}\n{self.secret}"
        hmac_code = hmac.new(
            string_to_sign.encode("utf-8"),
            digestmod=hashlib.sha256,
        ).digest()
        return base64.b64encode(hmac_code).decode("utf-8")
    
    def _build_message(self, notification: "AlertNotification") -> dict[str, Any]:
        """构建飞书消息体。
        
        使用 JSON 2.0 卡片格式，支持完整的 markdown 语法。
        """
        # 颜色映射
        color_map = {
            "info": "blue",
            "warning": "yellow",
            "error": "red",
            "critical": "red",
        }
        color = color_map.get(notification.severity.value, "grey")
        
        # 构建详情列表
        details = []
        
        # 基本信息
        details.append(f"**服务**: {notification.service_name or '未知'}")
        details.append(f"**来源**: {notification.source}")
        details.append(f"**类型**: {notification.event_type.value}")
        
        # 时间信息
        details.append(f"**时间**: {notification.last_timestamp.strftime('%Y-%m-%d %H:%M:%S')}")
        
        # 聚合信息
        if notification.count > 1:
            details.append(f"**触发次数**: {notification.count} 次")
        
        # Trace ID
        if notification.trace_ids:
            trace_str = ", ".join(notification.trace_ids[:3])
            if len(notification.trace_ids) > 3:
                trace_str += f" ... (共{len(notification.trace_ids)}个)"
            details.append(f"**Trace ID**: {trace_str}")
        
        # 元数据（排除 SQL 和堆栈，它们单独处理）
        sql_content: str | None = None
        stacktrace_content: str | None = None
        
        if notification.metadata:
            if "duration" in notification.metadata:
                details.append(f"**耗时**: {notification.metadata['duration']:.3f}s")
            if "endpoint" in notification.metadata:
                details.append(f"**接口**: {notification.metadata['endpoint']}")
            if "error_type" in notification.metadata:
                details.append(f"**错误类型**: {notification.metadata['error_type']}")
            if "error_message" in notification.metadata:
                details.append(f"**错误信息**: {notification.metadata['error_message']}")
            if "task_name" in notification.metadata:
                details.append(f"**任务**: {notification.metadata['task_name']}")
            
            # 事件循环阻塞检测专用字段
            if "blocked_ms" in notification.metadata:
                details.append(f"**阻塞时间**: {notification.metadata['blocked_ms']:.0f}ms")
            if "threshold_ms" in notification.metadata:
                details.append(f"**阈值**: {notification.metadata['threshold_ms']:.0f}ms")
            if "total_blocks" in notification.metadata:
                window_minutes = notification.metadata.get("window_minutes", 5)
                details.append(f"**近{window_minutes}分钟**: {notification.metadata['total_blocks']} 次")
            if "block_rate" in notification.metadata:
                details.append(f"**阻塞率**: {notification.metadata['block_rate']}")
            if "process_stats" in notification.metadata:
                stats = notification.metadata["process_stats"]
                if stats:
                    stats_str = f"CPU {stats.get('cpu_percent', 'N/A')}%, "
                    stats_str += f"RSS {stats.get('memory_rss_mb', 'N/A')}MB, "
                    stats_str += f"线程 {stats.get('num_threads', 'N/A')}"
                    details.append(f"**进程状态**: {stats_str}")
            
            # SQL 和堆栈单独处理
            if "sql" in notification.metadata:
                sql_content = notification.metadata["sql"]
            if "stacktrace" in notification.metadata:
                stacktrace_content = notification.metadata["stacktrace"]
        
        # 构建卡片元素
        elements: list[dict[str, Any]] = [
            {
                "tag": "markdown",
                "content": notification.message,
            },
            {
                "tag": "hr",
            },
            {
                "tag": "markdown",
                "content": "\n".join(details),
            },
        ]
        
        # 添加 SQL 代码块
        if sql_content:
            elements.append({"tag": "hr"})
            elements.append({
                "tag": "markdown",
                "content": f"**SQL**:\n```sql\n{sql_content}\n```",
            })
        
        # 添加堆栈代码块
        if stacktrace_content:
            elements.append({"tag": "hr"})
            elements.append({
                "tag": "markdown",
                "content": f"**堆栈**:\n```python\n{stacktrace_content}\n```",
            })
        
        # 构建卡片消息（飞书自定义机器人格式）
        card = {
            "msg_type": "interactive",
            "card": {
                "config": {
                    "wide_screen_mode": True,
                },
                "header": {
                    "template": color,
                    "title": {
                        "tag": "plain_text",
                        "content": notification.title,
                    },
                },
                "elements": elements,
            },
        }
        
        return card
    
    async def send(self, notification: "AlertNotification") -> bool:
        """发送飞书通知。"""
        try:
            import httpx
            
            # 构建消息
            message = self._build_message(notification)
            
            # 添加签名（如果配置了）
            if self.secret:
                timestamp = int(time.time())
                message["timestamp"] = str(timestamp)
                message["sign"] = self._generate_sign(timestamp)
            
            # 发送请求
            async with httpx.AsyncClient(timeout=10.0) as client:
                response = await client.post(self.webhook, json=message)
                result = response.json()
                
                if result.get("code") == 0 or result.get("StatusCode") == 0:
                    logger.debug(f"飞书通知发送成功: {notification.title}")
                    return True
                else:
                    logger.error(f"飞书通知发送失败: {result}")
                    return False
        except Exception as e:
            logger.error(f"飞书通知发送异常: {e}")
            return False


__all__ = ["FeishuNotifier"]



"""启动日志工具。

提供应用启动时的组件状态打印功能，包括 URL 脱敏。
"""

from __future__ import annotations

from dataclasses import dataclass, field
import re
from typing import Any

from aury.boot.common.logging import logger


@dataclass
class ComponentStatus:
    """组件状态。"""

    name: str
    status: str  # "ok", "error", "disabled"
    backend: str | None = None
    url: str | None = None
    details: dict[str, Any] = field(default_factory=dict)


def mask_url(url: str | None) -> str:
    """URL 脱敏（隐藏密码和敏感信息）。

    Args:
        url: 原始 URL

    Returns:
        脱敏后的 URL
    """
    if not url:
        return "N/A"

    # 匹配常见 URL 格式中的密码部分
    # redis://:password@host:port/db
    # amqp://user:password@host:port/vhost
    # postgresql://user:password@host:port/db
    patterns = [
        # user:password@ 格式
        (r"(://[^:]+:)([^@]+)(@)", r"\1***\3"),
        # :password@ 格式 (无用户名)
        (r"(://:)([^@]+)(@)", r"\1***\3"),
    ]

    masked = url
    for pattern, replacement in patterns:
        masked = re.sub(pattern, replacement, masked)

    return masked


def print_startup_banner(
    app_name: str = "Aury Boot",
    version: str = "",
    components: list[ComponentStatus] | None = None,
) -> None:
    """打印启动横幅和组件状态。

    Args:
        app_name: 应用名称
        version: 版本号
        components: 组件状态列表
    """
    # 打印横幅
    banner = f"""
╔══════════════════════════════════════════════════════════════╗
║  {app_name:^58}  ║
║  {f'v{version}' if version else '':^58}  ║
╚══════════════════════════════════════════════════════════════╝
"""
    logger.info(banner)

    # 打印组件状态
    if components:
        logger.info("组件状态:")
        logger.info("-" * 60)

        for comp in components:
            status_icon = "✓" if comp.status == "ok" else "✗" if comp.status == "error" else "○"
            status_text = f"[{status_icon}] {comp.name}"

            if comp.backend:
                status_text += f" ({comp.backend})"

            if comp.url:
                status_text += f" -> {mask_url(comp.url)}"

            if comp.status == "error" and comp.details.get("error"):
                status_text += f" | Error: {comp.details['error']}"

            logger.info(f"  {status_text}")

        logger.info("-" * 60)


def collect_component_status() -> list[ComponentStatus]:
    """收集所有组件状态。

    Returns:
        组件状态列表
    """
    from aury.boot.infrastructure.cache import CacheManager
    from aury.boot.infrastructure.database import DatabaseManager
    from aury.boot.infrastructure.scheduler import SchedulerManager
    from aury.boot.infrastructure.storage import StorageManager

    # 延迟导入新模块（可能不存在）
    try:
        from aury.boot.infrastructure.clients.redis import RedisClient
    except ImportError:
        RedisClient = None

    try:
        from aury.boot.infrastructure.channel import ChannelManager
    except ImportError:
        ChannelManager = None

    try:
        from aury.boot.infrastructure.mq import MQManager
    except ImportError:
        MQManager = None

    try:
        from aury.boot.infrastructure.events import EventBusManager
    except ImportError:
        EventBusManager = None

    statuses = []

    # Database - 收集所有实例
    for name, instance in DatabaseManager._instances.items():
        if instance.is_initialized:
            url = str(instance._engine.url) if instance._engine else None
            statuses.append(
                ComponentStatus(
                    name="Database" if name == "default" else f"Database [{name}]",
                    status="ok",
                    backend="SQLAlchemy",
                    url=url,
                )
            )

    # Cache - 收集所有实例
    for name, instance in CacheManager._instances.items():
        if instance._backend:
            statuses.append(
                ComponentStatus(
                    name="Cache" if name == "default" else f"Cache [{name}]",
                    status="ok",
                    backend=instance.backend_type,
                    url=(instance._config or {}).get("CACHE_URL"),
                )
            )

    # Storage - 收集所有实例
    for name, instance in StorageManager._instances.items():
        if instance._backend:
            backend_type = instance._config.backend.value if instance._config else "unknown"
            url = None
            if instance._config:
                if backend_type == "local":
                    url = instance._config.base_path
                elif instance._config.endpoint:
                    url = instance._config.endpoint
            statuses.append(
                ComponentStatus(
                    name="Storage" if name == "default" else f"Storage [{name}]",
                    status="ok",
                    backend=backend_type,
                    url=url,
                )
            )

    # Scheduler
    scheduler = SchedulerManager.get_instance()
    if scheduler._initialized:
        statuses.append(
            ComponentStatus(
                name="Scheduler",
                status="ok",
                backend="APScheduler",
            )
        )

    # Redis Clients
    if RedisClient:
        for name, instance in RedisClient._instances.items():
            if instance.is_initialized:
                statuses.append(
                    ComponentStatus(
                        name="Redis" if name == "default" else f"Redis [{name}]",
                        status="ok",
                        backend="redis",
                        url=instance._config.url if instance._config else None,
                    )
                )

    # Channel
    if ChannelManager:
        for name, instance in ChannelManager._instances.items():
            if instance.is_initialized:
                statuses.append(
                    ComponentStatus(
                        name="Channel" if name == "default" else f"Channel [{name}]",
                        status="ok",
                        backend=instance.backend_type,
                        url=getattr(instance._backend, "_url", None) if instance._backend else None,
                    )
                )

    # MQ
    if MQManager:
        for name, instance in MQManager._instances.items():
            if instance.is_initialized:
                statuses.append(
                    ComponentStatus(
                        name="MQ" if name == "default" else f"MQ [{name}]",
                        status="ok",
                        backend=instance.backend_type,
                        url=getattr(instance._backend, "_url", None) if instance._backend else None,
                    )
                )

    # Events
    if EventBusManager:
        for name, instance in EventBusManager._instances.items():
            if instance.is_initialized:
                statuses.append(
                    ComponentStatus(
                        name="Events" if name == "default" else f"Events [{name}]",
                        status="ok",
                        backend=instance.backend_type,
                        url=getattr(instance._backend, "_url", None) if instance._backend else None,
                    )
                )

    return statuses


__all__ = [
    "ComponentStatus",
    "collect_component_status",
    "mask_url",
    "print_startup_banner",
]



"""日志装饰器。

提供性能监控和异常日志装饰器。
"""

from __future__ import annotations

from collections.abc import Callable
from functools import wraps
import time
from typing import Any

from loguru import logger


def log_performance(threshold: float = 1.0) -> Callable:
    """性能监控装饰器。
    
    记录函数执行时间，超过阈值时警告。
    
    Args:
        threshold: 警告阈值（秒）
    
    使用示例:
        @log_performance(threshold=0.5)
        async def slow_operation():
            # 如果执行时间超过0.5秒，会记录警告
            pass
    """
    def decorator[T](func: Callable[..., T]) -> Callable[..., T]:
        @wraps(func)
        async def wrapper(*args, **kwargs) -> T:
            start_time = time.time()
            try:
                result = await func(*args, **kwargs)
                duration = time.time() - start_time
                
                if duration > threshold:
                    logger.warning(
                        f"性能警告: {func.__module__}.{func.__name__} 执行耗时 {duration:.3f}s "
                        f"(阈值: {threshold}s)"
                    )
                else:
                    logger.debug(
                        f"性能: {func.__module__}.{func.__name__} 执行耗时 {duration:.3f}s"
                    )
                
                return result
            except Exception as exc:
                duration = time.time() - start_time
                logger.error(
                    f"执行失败: {func.__module__}.{func.__name__} | "
                    f"耗时: {duration:.3f}s | "
                    f"异常: {type(exc).__name__}: {exc}"
                )
                raise
        
        return wrapper
    return decorator


def log_exceptions[T](func: Callable[..., T]) -> Callable[..., T]:
    """异常日志装饰器。
    
    自动记录函数抛出的异常。
    
    使用示例:
        @log_exceptions
        async def risky_operation():
            # 如果抛出异常，会自动记录
            pass
    """
    @wraps(func)
    async def wrapper(*args, **kwargs) -> T:
        try:
            return await func(*args, **kwargs)
        except Exception as exc:
            logger.exception(
                f"异常捕获: {func.__module__}.{func.__name__} | "
                f"参数: args={args}, kwargs={kwargs} | "
                f"异常: {type(exc).__name__}: {exc}"
            )
            raise
    
    return wrapper


def get_class_logger(obj: object) -> Any:
    """获取类专用的日志器（函数式工具函数）。
    
    根据对象的类和模块名创建绑定的日志器。
    
    Args:
        obj: 对象实例或类
        
    Returns:
        绑定的日志器实例
        
    使用示例:
        class MyService:
            def do_something(self):
                log = get_class_logger(self)
                log.info("执行操作")
    """
    if isinstance(obj, type):
        class_name = obj.__name__
        module_name = obj.__module__
    else:
        class_name = obj.__class__.__name__
        module_name = obj.__class__.__module__
    return logger.bind(name=f"{module_name}.{class_name}")


__all__ = [
    "get_class_logger",
    "log_exceptions",
    "log_performance",
]



"""数据库连接追踪器 - 对标 HikariCP leakDetectionThreshold。

用于检测连接泄漏，追踪连接的获取来源代码位置。

功能：
    - 连接 checkout/checkin 事件追踪
    - 记录获取连接时的调用栈
    - 检测持有时间过长的连接
    - 支持多种数据库的系统表查询（PostgreSQL、MySQL、SQLite）

配置项（环境变量前缀 DATABASE__TRACKER__）：
    DATABASE__TRACKER__ENABLED: 是否启用追踪（默认 False，生产环境建议关闭）
    DATABASE__TRACKER__LEAK_DETECTION_THRESHOLD: 泄漏检测阈值（秒），默认 300
    DATABASE__TRACKER__MAX_IDLE_CONNECTIONS: idle 连接数报警阈值
    DATABASE__TRACKER__MAX_TOTAL_CONNECTIONS: 总连接数报警阈值
"""

from __future__ import annotations

import asyncio
import time
import traceback
from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import TYPE_CHECKING, Any

from sqlalchemy import event, text
from sqlalchemy.ext.asyncio import AsyncSession

from aury.boot.common.logging import logger

if TYPE_CHECKING:
    from sqlalchemy.pool import Pool


@dataclass
class ConnectionTrace:
    """连接追踪信息。"""

    conn_id: int  # 连接标识（id(dbapi_conn)）
    checkout_time: float
    stack_trace: str  # 完整调用栈
    code_location: str  # 简化的代码位置
    checkout_task: str | None = None  # asyncio task name


class ConnectionTracker:
    """
    连接追踪器单例。

    通过 SQLAlchemy pool 事件追踪每个连接的获取来源。
    类似 HikariCP 的 leakDetectionThreshold 功能。
    """

    _instance: ConnectionTracker | None = None

    def __init__(self, enabled: bool = False, app_package: str = "app") -> None:
        """
        初始化连接追踪器。

        Args:
            enabled: 是否启用追踪
            app_package: 应用包名，用于过滤调用栈（只保留应用代码）
        """
        # conn_id -> ConnectionTrace
        self._traces: dict[int, ConnectionTrace] = {}
        self._enabled = enabled
        self._app_package = app_package

    @classmethod
    def get_instance(cls) -> ConnectionTracker:
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance

    @classmethod
    def initialize(cls, enabled: bool = False, app_package: str = "app") -> ConnectionTracker:
        """初始化并返回实例。"""
        cls._instance = cls(enabled=enabled, app_package=app_package)
        return cls._instance

    @property
    def enabled(self) -> bool:
        return self._enabled

    def _extract_code_location(self, stack: str) -> str:
        """从调用栈提取关键代码位置（过滤掉框架代码）。"""
        lines = stack.strip().split("\n")
        relevant_lines = []
        app_pkg = f"/{self._app_package}/"

        for i, line in enumerate(lines):
            # 只保留应用目录下的代码
            if app_pkg in line and "connection_tracker" not in line:
                if line.strip().startswith("File"):
                    relevant_lines.append(line.strip())
                    # 也获取下一行（代码内容）
                    if i + 1 < len(lines):
                        relevant_lines.append(lines[i + 1].strip())

        return "\n".join(relevant_lines[-6:]) if relevant_lines else "Unknown"

    def on_checkout(self, dbapi_conn: Any, connection_record: Any, connection_proxy: Any) -> None:
        """连接被取出时记录。"""
        if not self._enabled:
            return

        try:
            conn_id = id(dbapi_conn)

            # 获取调用栈
            stack = "".join(traceback.format_stack())
            code_location = self._extract_code_location(stack)

            # 获取当前 asyncio task 名称
            task_name = None
            try:
                task = asyncio.current_task()
                if task:
                    task_name = task.get_name()
            except RuntimeError:
                pass

            self._traces[conn_id] = ConnectionTrace(
                conn_id=conn_id,
                checkout_time=time.time(),
                stack_trace=stack,
                code_location=code_location,
                checkout_task=task_name,
            )
        except Exception as e:
            logger.warning(f"Failed to track connection checkout: {e}")

    def on_checkin(self, dbapi_conn: Any, connection_record: Any) -> None:
        """连接归还时清理。"""
        if not self._enabled:
            return

        try:
            conn_id = id(dbapi_conn)
            self._traces.pop(conn_id, None)
        except Exception:
            pass

    def get_trace(self, conn_id: int) -> ConnectionTrace | None:
        """获取指定连接的追踪信息。"""
        return self._traces.get(conn_id)

    def get_all_traces(self) -> dict[int, ConnectionTrace]:
        """获取所有追踪信息。"""
        return dict(self._traces)

    def get_long_held_connections(self, threshold_seconds: float = 300) -> list[dict]:
        """
        获取持有时间过长的连接（疑似泄漏）。

        类似 HikariCP 的 leakDetectionThreshold。

        Args:
            threshold_seconds: 持有时间阈值（秒）

        Returns:
            疑似泄漏的连接列表，包含持有时间和代码位置
        """
        now = time.time()
        result = []
        for conn_id, trace in self._traces.items():
            held_seconds = now - trace.checkout_time
            if held_seconds > threshold_seconds:
                result.append(
                    {
                        "conn_id": conn_id,
                        "held_seconds": held_seconds,
                        "code_location": trace.code_location,
                        "task_name": trace.checkout_task,
                        "full_stack": trace.stack_trace,
                    }
                )
        return result


def setup_connection_tracking(pool: Pool, enabled: bool = False, app_package: str = "app") -> ConnectionTracker:
    """
    设置连接追踪。

    在应用启动时调用，注册 SQLAlchemy pool 事件。

    Args:
        pool: SQLAlchemy 连接池（sync_engine.pool）
        enabled: 是否启用追踪
        app_package: 应用包名

    Returns:
        ConnectionTracker 实例
    """
    tracker = ConnectionTracker.initialize(enabled=enabled, app_package=app_package)

    if not enabled:
        logger.info("Database connection tracking is disabled")
        return tracker

    event.listen(pool, "checkout", tracker.on_checkout)
    event.listen(pool, "checkin", tracker.on_checkin)

    logger.info("Database connection tracking enabled (like HikariCP leakDetectionThreshold)")
    return tracker


# ==================== 多数据库健康检查 ====================


class DbHealthChecker(ABC):
    """数据库健康检查抽象基类。"""

    @abstractmethod
    async def get_connection_stats(self, session: AsyncSession) -> dict:
        """获取连接统计信息。"""
        pass

    @abstractmethod
    async def get_idle_connections(
        self, session: AsyncSession, idle_threshold_seconds: int = 300
    ) -> list[dict]:
        """获取空闲连接列表。"""
        pass

    @abstractmethod
    async def get_connection_by_id(self, session: AsyncSession, conn_id: Any) -> dict | None:
        """根据连接标识获取连接信息。"""
        pass


class PostgreSQLHealthChecker(DbHealthChecker):
    """PostgreSQL 健康检查实现。"""

    async def get_connection_stats(self, session: AsyncSession) -> dict:
        """获取连接统计信息。"""
        sql = text("""
            SELECT state, COUNT(*) AS count
            FROM pg_stat_activity
            WHERE datname = current_database()
            GROUP BY state
        """)
        result = await session.execute(sql)
        stats = {row.state: row.count for row in result.fetchall()}
        return {
            "total": sum(stats.values()),
            "stats": stats,
        }

    async def get_idle_connections(
        self, session: AsyncSession, idle_threshold_seconds: int = 300
    ) -> list[dict]:
        """获取空闲连接列表。"""
        sql = text(f"""
            SELECT 
                pid,
                usename,
                application_name,
                state,
                EXTRACT(EPOCH FROM (NOW() - state_change)) AS idle_seconds,
                EXTRACT(EPOCH FROM (NOW() - backend_start)) AS connection_age_seconds,
                LEFT(query, 300) AS last_query,
                client_addr
            FROM pg_stat_activity
            WHERE datname = current_database()
              AND state = 'idle'
              AND NOW() - state_change > INTERVAL '{idle_threshold_seconds} seconds'
            ORDER BY state_change
            LIMIT 30
        """)
        result = await session.execute(sql)
        return [dict(row._mapping) for row in result.fetchall()]

    async def get_connection_by_id(self, session: AsyncSession, conn_id: Any) -> dict | None:
        """根据 PID 获取连接信息。"""
        sql = text("""
            SELECT 
                pid,
                usename,
                application_name,
                state,
                query,
                backend_start,
                state_change
            FROM pg_stat_activity
            WHERE pid = :pid
        """)
        result = await session.execute(sql, {"pid": conn_id})
        row = result.fetchone()
        return dict(row._mapping) if row else None


class MySQLHealthChecker(DbHealthChecker):
    """MySQL 健康检查实现。"""

    async def get_connection_stats(self, session: AsyncSession) -> dict:
        """获取连接统计信息。"""
        sql = text("SHOW STATUS LIKE 'Threads_connected'")
        result = await session.execute(sql)
        row = result.fetchone()
        total = int(row[1]) if row else 0

        # 获取各状态统计
        sql = text("""
            SELECT COMMAND AS state, COUNT(*) AS count
            FROM information_schema.PROCESSLIST
            GROUP BY COMMAND
        """)
        result = await session.execute(sql)
        stats = {row.state: row.count for row in result.fetchall()}

        return {
            "total": total,
            "stats": stats,
        }

    async def get_idle_connections(
        self, session: AsyncSession, idle_threshold_seconds: int = 300
    ) -> list[dict]:
        """获取空闲连接列表。"""
        sql = text(f"""
            SELECT 
                ID AS pid,
                USER AS usename,
                HOST AS client_addr,
                DB AS database,
                COMMAND AS state,
                TIME AS idle_seconds,
                LEFT(INFO, 300) AS last_query
            FROM information_schema.PROCESSLIST
            WHERE COMMAND = 'Sleep'
              AND TIME > {idle_threshold_seconds}
            ORDER BY TIME DESC
            LIMIT 30
        """)
        result = await session.execute(sql)
        return [dict(row._mapping) for row in result.fetchall()]

    async def get_connection_by_id(self, session: AsyncSession, conn_id: Any) -> dict | None:
        """根据连接 ID 获取连接信息。"""
        sql = text("""
            SELECT 
                ID AS pid,
                USER AS usename,
                HOST AS client_addr,
                DB AS database,
                COMMAND AS state,
                TIME AS idle_seconds,
                INFO AS query
            FROM information_schema.PROCESSLIST
            WHERE ID = :conn_id
        """)
        result = await session.execute(sql, {"conn_id": conn_id})
        row = result.fetchone()
        return dict(row._mapping) if row else None


class SQLiteHealthChecker(DbHealthChecker):
    """SQLite 健康检查实现（功能有限）。"""

    async def get_connection_stats(self, session: AsyncSession) -> dict:
        """SQLite 是单文件数据库，连接统计有限。"""
        return {
            "total": 1,
            "stats": {"active": 1},
            "note": "SQLite is a single-file database with limited connection stats",
        }

    async def get_idle_connections(
        self, session: AsyncSession, idle_threshold_seconds: int = 300
    ) -> list[dict]:
        """SQLite 不支持连接级别的空闲检测。"""
        return []

    async def get_connection_by_id(self, session: AsyncSession, conn_id: Any) -> dict | None:
        """SQLite 不支持按 ID 查询连接。"""
        return None


def get_health_checker(database_url: str) -> DbHealthChecker:
    """
    根据数据库 URL 获取对应的健康检查器。

    Args:
        database_url: 数据库连接 URL

    Returns:
        对应数据库类型的健康检查器
    """
    url_lower = database_url.lower()
    if "postgresql" in url_lower or "postgres" in url_lower:
        return PostgreSQLHealthChecker()
    elif "mysql" in url_lower or "mariadb" in url_lower:
        return MySQLHealthChecker()
    elif "sqlite" in url_lower:
        return SQLiteHealthChecker()
    else:
        # 默认使用 PostgreSQL（最常用）
        logger.warning(f"Unknown database type in URL, using PostgreSQL health checker")
        return PostgreSQLHealthChecker()


__all__ = [
    "ConnectionTrace",
    "ConnectionTracker",
    "DbHealthChecker",
    "MySQLHealthChecker",
    "PostgreSQLHealthChecker",
    "SQLiteHealthChecker",
    "get_health_checker",
    "setup_connection_tracking",
]



"""统一的迁移配置初始化模块。

这个模块提供单一的数据源来创建迁移配置，被 init 命令和 MigrationManager 共同使用。
"""

from pathlib import Path


def ensure_migration_setup(
    base_path: Path,
    config_path: str,
    script_location: str,
    model_modules: list[str],
) -> None:
    """确保迁移配置和目录存在，不存在则自动创建。
    
    Args:
        base_path: 项目根目录
        config_path: alembic.ini 路径
        script_location: 迁移脚本目录名
        model_modules: 模型模块列表
    """
    script_dir = base_path / script_location
    config_file = base_path / config_path
    
    # 创建迁移脚本目录
    if not script_dir.exists():
        script_dir.mkdir(parents=True, exist_ok=True)
        
        # 创建 versions 目录
        versions_dir = script_dir / "versions"
        versions_dir.mkdir(exist_ok=True)
        
        # 创建 env.py
        env_py = script_dir / "env.py"
        if not env_py.exists():
            env_content = _get_env_py_template(model_modules)
            env_py.write_text(env_content, encoding="utf-8")
        
        # 创建 script.py.mako
        mako_file = script_dir / "script.py.mako"
        if not mako_file.exists():
            mako_content = _get_script_mako_template()
            mako_file.write_text(mako_content, encoding="utf-8")
    
    # 创建 alembic.ini
    if not config_file.exists():
        ini_content = _get_alembic_ini_template(script_location)
        config_file.write_text(ini_content, encoding="utf-8")


def _get_env_py_template(model_modules: list[str]) -> str:
    """获取 env.py 模板（异步版本）。"""
    model_modules_str = repr(model_modules)
    
    return f'''"""Alembic 环境配置（异步）。

由 Aury Boot 自动生成，并改造为全异步模式，
适配 sqlite+aiosqlite / postgresql+asyncpg / mysql+asyncmy 等异步驱动。
"""

from logging.config import fileConfig
from pathlib import Path
import os
import sys

from alembic import context
from sqlalchemy import pool
from sqlalchemy.ext.asyncio import async_engine_from_config

# 导入模型基类
from aury.boot.domain.models import Base

# Alembic Config 对象
config = context.config

# 解析日志配置
if config.config_file_name is not None:
    fileConfig(config.config_file_name)

# === 模型加载（基于项目包名自动发现） ===
# 确保项目根目录在 sys.path 中
project_root = Path(__file__).resolve().parents[1]
if str(project_root) not in sys.path:
    sys.path.insert(0, str(project_root))

from aury.boot.application.migrations import load_all_models
try:
    from aury.boot.commands.config import get_project_config
    _cfg = get_project_config()
    if _cfg.has_package:
        _model_modules = [f"{{_cfg.package}}.models", f"{{_cfg.package}}.**.models"]
    else:
        _model_modules = ["models"]
except Exception:
    _model_modules = {model_modules_str}

# 加载模型，确保 Base.metadata 完整
load_all_models(_model_modules)

# 目标元数据
target_metadata = Base.metadata


# === 兼容性处理 ===
# 过滤 PostgreSQL 15+ 特有的约束参数，确保生成的 migration 兼容旧版本 PG
_PG15_CONSTRAINT_KWARGS = {{"postgresql_nulls_not_distinct", "postgresql_include"}}

def _render_item(type_, obj, autogen_context):
    """自定义渲染，过滤不兼容的约束参数。"""
    if type_ == "unique_constraint" and hasattr(obj, "kwargs"):
        for key in _PG15_CONSTRAINT_KWARGS:
            obj.kwargs.pop(key, None)
    return False  # 使用默认渲染


def get_url() -> str:
    """获取数据库 URL（优先环境变量，其次 alembic.ini）。"""
    return os.environ.get("DATABASE_URL") or config.get_main_option("sqlalchemy.url", "")


def run_migrations_offline() -> None:
    """离线模式运行迁移（不建立实际连接）。"""
    url = get_url()
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={{"paramstyle": "named"}},
        compare_type=True,
        compare_server_default=True,
        render_as_batch=True,
        render_item=_render_item,
    )
    with context.begin_transaction():
        context.run_migrations()


def _do_run_migrations(connection) -> None:
    """在同步上下文里执行迁移（由 AsyncConnection.run_sync 调用）。"""
    context.configure(
        connection=connection,
        target_metadata=target_metadata,
        compare_type=True,
        compare_server_default=True,
        render_as_batch=True,
        render_item=_render_item,
    )
    with context.begin_transaction():
        context.run_migrations()


async def _run_async_migrations() -> None:
    configuration = config.get_section(config.config_ini_section, {{}})
    configuration["sqlalchemy.url"] = get_url()

    connectable = async_engine_from_config(
        configuration,
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )
    async with connectable.connect() as connection:
        await connection.run_sync(_do_run_migrations)
    await connectable.dispose()


def run_migrations_online() -> None:
    import asyncio
    asyncio.run(_run_async_migrations())


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()
'''


def _get_script_mako_template() -> str:
    """获取 script.py.mako 模板。"""
    return '''"""${message}

Revision ID: ${up_revision}
Revises: ${down_revision | comma,n}
Create Date: ${create_date}
"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
${imports if imports else ""}

revision: str = ${repr(up_revision)}
down_revision: Union[str, None] = ${repr(down_revision)}
branch_labels: Union[str, Sequence[str], None] = ${repr(branch_labels)}
depends_on: Union[str, Sequence[str], None] = ${repr(depends_on)}


def upgrade() -> None:
    ${upgrades if upgrades else "pass"}


def downgrade() -> None:
    ${downgrades if downgrades else "pass"}
'''


def _get_alembic_ini_template(script_location: str) -> str:
    """获取 alembic.ini 模板。"""
    return f'''[alembic]
script_location = {script_location}
file_template = %%(year)d%%(month).2d%%(day).2d_%%(hour).2d%%(minute).2d_%%(rev)s_%%(slug)s
timezone = UTC
truncate_slug_length = 40
version_path_separator = os

[loggers]
keys = root,sqlalchemy,alembic

[handlers]
keys = console

[formatters]
keys = generic

[logger_root]
level = WARN
handlers = console
qualname =

[logger_sqlalchemy]
level = WARN
handlers =
qualname = sqlalchemy.engine

[logger_alembic]
level = INFO
handlers =
qualname = alembic

[handler_console]
class = StreamHandler
args = (sys.stderr,)
level = NOTSET
formatter = generic

[formatter_generic]
format = %(levelname)-5.5s [%(name)s] %(message)s
datefmt = %H:%M:%S
'''


__all__ = ["ensure_migration_setup"]
